@@ -17,10 +17,10 @@
 #ifndef SW_CLIENT_H_
 #define SW_CLIENT_H_
 
-SW_EXTERN_C_BEGIN
+#include "swoole_api.h"
+#include "ssl.h"
 
-#include "buffer.h"
-#include "connection.h"
+SW_EXTERN_C_BEGIN
 
 #define SW_SOCK_ASYNC    1
 #define SW_SOCK_SYNC     0
@@ -63,7 +63,7 @@ typedef struct _swClient
     int _sock_type;
     int _sock_domain;
     int _protocol;
-    int reactor_fdtype;
+    enum swFd_type reactor_fdtype;
 
     uchar active :1;
     uchar async :1;
@@ -122,7 +122,7 @@ typedef struct _swClient
 
     swString *buffer;
     uint32_t wait_length;
-    uint32_t buffer_input_size;
+    uint32_t input_buffer_size;
 
     uint32_t buffer_high_watermark;
     uint32_t buffer_low_watermark;
@@ -136,7 +136,7 @@ typedef struct _swClient
 
     void (*onConnect)(struct _swClient *cli);
     void (*onError)(struct _swClient *cli);
-    void (*onReceive)(struct _swClient *cli, char *data, uint32_t length);
+    void (*onReceive)(struct _swClient *cli, const char *data, uint32_t length);
     void (*onClose)(struct _swClient *cli);
     void (*onBufferFull)(struct _swClient *cli);
     void (*onBufferEmpty)(struct _swClient *cli);
@@ -150,7 +150,7 @@ typedef struct _swClient
 } swClient;
 
 void swClient_init_reactor(swReactor *reactor);
-int swClient_create(swClient *cli, int type, int async);
+int swClient_create(swClient *cli, enum swSocket_type type, int async);
 int swClient_sleep(swClient *cli);
 int swClient_wakeup(swClient *cli);
 int swClient_shutdown(swClient *cli, int __how);
@@ -166,15 +166,15 @@ typedef struct _swStream
     swString *buffer;
     uint8_t cancel;
     void *private_data;
-    void (*response)(struct _swStream *stream, char *data, uint32_t length);
+    void (*response)(struct _swStream *stream, const char *data, uint32_t length);
     swClient client;
 } swStream;
 
-swStream* swStream_new(char *dst_host, int dst_port, int type);
-int swStream_send(swStream *stream, char *data, size_t length);
+swStream* swStream_new(const char *dst_host, int dst_port, enum swSocket_type type);
+int swStream_send(swStream *stream, const char *data, size_t length);
 void swStream_set_protocol(swProtocol *protocol);
 void swStream_set_max_length(swStream *stream, uint32_t max_length);
-int swStream_recv_blocking(int fd, void *__buf, size_t __len);
+int swStream_recv_blocking(swSocket *sock, void *__buf, size_t __len);
 //----------------------------------------Stream End------------------------------------
 
 SW_EXTERN_C_END
