@@ -11,6 +11,7 @@
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
   | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  |         Twosee  <twose@qq.com>                                       |
   +----------------------------------------------------------------------+
 */
 
@@ -22,6 +23,7 @@
 
 #include <limits.h>
 
+#include <functional>
 #include <string>
 #include <unordered_map>
 
@@ -56,12 +58,14 @@ struct socket_poll_fd
     int16_t events;
     int16_t revents;
     void *ptr;
+    swSocket *socket;
 
     socket_poll_fd(int16_t _event, void *_ptr)
     {
         events = _event;
         ptr = _ptr;
         revents = 0;
+        socket = nullptr;
     }
 };
 
@@ -79,6 +83,11 @@ public:
         return state;
     }
 
+    inline long get_init_msec()
+    {
+        return init_msec;
+    }
+
     inline long get_cid()
     {
         return cid;
@@ -182,6 +191,12 @@ public:
         return peak_num;
     }
 
+    static inline long get_elapsed(long cid)
+    {
+        Coroutine *co = cid == 0 ? get_current() : get_by_cid(cid);
+        return sw_likely(co) ? swTimer_get_absolute_msec() - co->get_init_msec() : -1;
+    }
+
     static void print_list();
 
 protected:
@@ -196,6 +211,7 @@ protected:
 
     sw_coro_state state = SW_CORO_INIT;
     long cid;
+    long init_msec = swTimer_get_absolute_msec();
     void *task = nullptr;
     Context ctx;
     Coroutine *origin;
@@ -242,6 +258,7 @@ protected:
 namespace coroutine
 {
 bool async(swAio_handler handler, swAio_event &event, double timeout = -1);
+bool async(const std::function<void(void)> &fn, double timeout = -1);
 }
 //-------------------------------------------------------------------------------
 }
