@@ -26,13 +26,17 @@ extern "C"
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/statvfs.h>
+#include <stdint.h>
 #include <poll.h>
 
 /**
  * base
  */
 uint8_t swoole_coroutine_is_in();
-
+long swoole_coroutine_get_current_id();
+void swoole_coroutine_sleep(int sec);
+void swoole_coroutine_usleep(int usec);
 /**
  * file
  */
@@ -47,6 +51,8 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode);
 int swoole_coroutine_rmdir(const char *pathname);
 int swoole_coroutine_rename(const char *oldpath, const char *newpath);
 int swoole_coroutine_flock(int fd, int operation);
+int swoole_coroutine_flock_ex(char *filename, int fd, int operation);
+int swoole_coroutine_statvfs(const char *path, struct statvfs *buf);
 
 /**
  * socket
@@ -59,11 +65,13 @@ ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags);
 int swoole_coroutine_close(int fd);
 int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen);
 int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout);
+int swoole_coroutine_socket_set_timeout(int fd, int which, double timeout);
+int swoole_coroutine_socket_wait_event(int fd, int event, double timeout);
 
 /**
  * wait
  */
-void swoole_coroutine_signal_init();
+size_t swoole_coroutine_wait_count();
 pid_t swoole_coroutine_waitpid(pid_t __pid, int *__stat_loc, int __options);
 pid_t swoole_coroutine_wait(int *__stat_loc);
 
