@@ -34,13 +34,14 @@ SW_API uchar swoole_timer_clear(long timer_id);
 SW_API void swoole_timer_free();
 
 SW_API int swoole_event_init();
-SW_API int swoole_event_add(int fd, int events, int fdtype);
-SW_API int swoole_event_set(int fd, int events, int fdtype);
-SW_API int swoole_event_del(int fd);
+SW_API int swoole_event_add(swSocket *socket, int events);
+SW_API int swoole_event_set(swSocket *socket, int events);
+SW_API int swoole_event_del(swSocket *socket);
 SW_API void swoole_event_defer(swCallback cb, void *private_data);
-SW_API int swoole_event_write(int fd, const void *data, size_t len);
+SW_API int swoole_event_write(swSocket *socket, const void *data, size_t len);
 SW_API int swoole_event_wait();
 SW_API int swoole_event_free();
+SW_API int swoole_event_set_handler(int fdtype, swReactor_handler handler);
 
 #ifdef __MACH__
 swReactor* sw_reactor();
