@@ -48,7 +48,7 @@
 #else
 #define SW_SOCKET_BUFFER_SIZE      8388608
 #endif
-#define SW_SOCKET_SEND_TIMEOUT     1.0
+#define SW_SOCKET_SEND_TIMEOUT     60.0
 #define SW_SOCKET_ARRAY_INIT_SIZE  1024
 #define SW_SOCKET_SYNC_SEND_RETRY_COUNT  10
 
@@ -110,6 +110,7 @@
 #define SW_WORKER_USE_SIGNALFD           1
 #define SW_WORKER_MAX_WAIT_TIME          3
 #define SW_WORKER_MIN_REQUEST            10
+#define SW_WORKER_MAX_RECV_CHUNK_COUNT   32
 
 #define SW_REACTOR_MAXEVENTS             4096
 #define SW_SESSION_LIST_SIZE             (1*1024*1024)
@@ -117,6 +118,8 @@
 #define SW_MSGMAX                        65536
 #define SW_UNIXSOCK_MAX_BUF_SIZE         (2*1024*1024)
 
+#define SW_DGRAM_HEADER_SIZE                32
+
 /**
  * The maximum number of Reactor threads
  * the number of the CPU cores threads will be started by default
@@ -141,8 +144,8 @@
 /**
  * ringbuffer memory pool size
  */
-#define SW_BUFFER_OUTPUT_SIZE            (2*1024*1024)
-#define SW_BUFFER_INPUT_SIZE             (2*1024*1024)
+#define SW_OUTPUT_BUFFER_SIZE            (2*1024*1024)
+#define SW_INPUT_BUFFER_SIZE             (2*1024*1024)
 #define SW_BUFFER_MIN_SIZE               65536
 #define SW_SEND_BUFFER_SIZE              65536
 
@@ -170,6 +173,7 @@
 
 #define SW_SSL_BUFFER_SIZE               16384
 #define SW_SSL_CIPHER_LIST               "EECDH+AESGCM:EDH+AESGCM:AES256+EECDH:AES256+EDH"
+#define SW_SSL_ECDH_CURVE                "auto"
 #define SW_SSL_NPN_ADVERTISE             "\x08http/1.1"
 #define SW_SSL_HTTP2_NPN_ADVERTISE       "\x02h2"
 
