@@ -50,9 +50,7 @@
 #define SW_HAVE_COUNTABLE 1
 #endif
 
-#include "swoole.h"
-#include "server.h"
-#include "client.h"
+#include "swoole_api.h"
 #include "async.h"
 
 #ifdef SW_HAVE_ZLIB
@@ -141,48 +139,6 @@ enum php_swoole_client_callback_type
     SW_CLIENT_CB_onSSLReady,
 #endif
 };
-//--------------------------------------------------------
-enum php_swoole_server_callback_type
-{
-    SW_SERVER_CB_onStart,          //master
-    SW_SERVER_CB_onShutdown,       //master
-    SW_SERVER_CB_onWorkerStart,    //worker(event & task)
-    SW_SERVER_CB_onWorkerStop,     //worker(event & task)
-    SW_SERVER_CB_onTask,           //worker(task)
-    SW_SERVER_CB_onFinish,         //worker(event & task)
-    SW_SERVER_CB_onWorkerExit,     //worker(event)
-    SW_SERVER_CB_onWorkerError,    //manager
-    SW_SERVER_CB_onManagerStart,   //manager
-    SW_SERVER_CB_onManagerStop,    //manager
-    SW_SERVER_CB_onPipeMessage,    //worker(event & task)
-};
-//--------------------------------------------------------
-enum php_swoole_server_port_callback_type
-{
-    SW_SERVER_CB_onConnect,        //worker(event)
-    SW_SERVER_CB_onReceive,        //worker(event)
-    SW_SERVER_CB_onClose,          //worker(event)
-    SW_SERVER_CB_onPacket,         //worker(event)
-    SW_SERVER_CB_onRequest,        //http server
-    SW_SERVER_CB_onHandShake,      //worker(event)
-    SW_SERVER_CB_onOpen,           //worker(event)
-    SW_SERVER_CB_onMessage,        //worker(event)
-    SW_SERVER_CB_onBufferFull,     //worker(event)
-    SW_SERVER_CB_onBufferEmpty,    //worker(event)
-};
-
-#define PHP_SWOOLE_SERVER_CALLBACK_NUM         (SW_SERVER_CB_onPipeMessage + 1)
-#define PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM    (SW_SERVER_CB_onBufferEmpty + 1)
-
-typedef struct
-{
-    zval *callbacks[PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM];
-    zend_fcall_info_cache *caches[PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM];
-    zval _callbacks[PHP_SWOOLE_SERVER_PORT_CALLBACK_NUM];
-    swServer *serv;
-    swListenPort *port;
-    zval *zsetting;
-} php_swoole_server_port_property;
 //---------------------------------------------------------
 #define SW_FLAG_KEEP                        (1u << 12)
 #define SW_FLAG_ASYNC                       (1u << 10)
@@ -221,9 +177,11 @@ typedef struct
     zend_fcall_info_cache fci_cache;
 } php_swoole_fci;
 //---------------------------------------------------------
-#define php_swoole_socktype(type)           (type & (~SW_FLAG_SYNC) & (~SW_FLAG_ASYNC) & (~SW_FLAG_KEEP) & (~SW_SOCK_SSL))
 
-#define SW_LONG_CONNECTION_KEY_LEN          64
+static sw_inline enum swSocket_type php_swoole_socktype(long type)
+{
+    return (enum swSocket_type) (type & (~SW_FLAG_SYNC) & (~SW_FLAG_ASYNC) & (~SW_FLAG_KEEP) & (~SW_SOCK_SSL));
+}
 
 extern zend_class_entry *swoole_event_ce;
 extern zend_class_entry *swoole_timer_ce;
@@ -232,7 +190,6 @@ extern zend_class_entry *swoole_client_ce;
 extern zend_class_entry *swoole_server_ce;
 extern zend_object_handlers swoole_server_handlers;
 extern zend_class_entry *swoole_connection_iterator_ce;
-extern zend_class_entry *swoole_buffer_ce;
 extern zend_class_entry *swoole_process_ce;
 extern zend_class_entry *swoole_http_server_ce;
 extern zend_object_handlers swoole_http_server_handlers;
@@ -291,7 +248,6 @@ PHP_FUNCTION(swoole_last_error);
 void php_swoole_event_minit(int module_number);
 // base
 void php_swoole_atomic_minit(int module_number);
-void php_swoole_buffer_minit(int module_number);
 void php_swoole_lock_minit(int module_number);
 void php_swoole_process_minit(int module_number);
 void php_swoole_process_pool_minit(int module_number);
@@ -338,6 +294,7 @@ void php_swoole_process_clean();
 int php_swoole_process_start(swWorker *process, zval *zobject);
 
 int php_swoole_reactor_init();
+void php_swoole_set_global_option(HashTable *vht);
 
 // shutdown
 void php_swoole_register_shutdown_function(const char *function);
@@ -349,19 +306,18 @@ void php_swoole_event_init();
 void php_swoole_event_wait();
 void php_swoole_event_exit();
 
-void php_swoole_server_register_callbacks(swServer *serv);
-void php_swoole_client_free(zval *zobject, swClient *cli);
-swClient* php_swoole_client_new(zval *zobject, char *host, int host_len, int port);
-void php_swoole_client_check_setting(swClient *cli, zval *zset);
-#ifdef SW_USE_OPENSSL
-void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset);
-#endif
-
 static sw_inline zend_bool php_swoole_websocket_frame_is_object(zval *zdata)
 {
     return Z_TYPE_P(zdata) == IS_OBJECT && instanceof_function(Z_OBJCE_P(zdata), swoole_websocket_frame_ce);
 }
 
+static sw_inline size_t php_swoole_get_send_data(zval *zdata, char **str)
+{
+    convert_to_string(zdata);
+    *str = Z_STRVAL_P(zdata);
+    return Z_STRLEN_P(zdata);
+}
+
 #ifdef SW_HAVE_ZLIB
 #define php_swoole_websocket_frame_pack        php_swoole_websocket_frame_pack_ex
 #define php_swoole_websocket_frame_object_pack php_swoole_websocket_frame_object_pack_ex
@@ -391,28 +347,10 @@ php_socket *swoole_convert_to_socket(int sock);
 void swoole_php_socket_free(zval *zsocket);
 #endif
 
-zend_fcall_info_cache* php_swoole_server_get_fci_cache(swServer *serv, int server_fd, int event_type);
-void php_swoole_server_before_start(swServer *serv, zval *zobject);
-void php_swoole_http_server_init_global_variant();
-void php_swoole_server_send_yield(swServer *serv, int fd, zval *zdata, zval *return_value);
-void php_swoole_get_recv_data(swServer *serv, zval *zdata, swEventData *req, char *header, uint32_t header_length);
-size_t php_swoole_get_send_data(zval *zdata, char **str);
-void php_swoole_onConnect(swServer *, swDataHead *);
-int php_swoole_onReceive(swServer *, swEventData *);
-int php_swoole_http_onReceive(swServer *, swEventData *);
-void php_swoole_http_onClose(swServer *, swDataHead *);
-int php_swoole_onPacket(swServer *, swEventData *);
-void php_swoole_onClose(swServer *, swDataHead *);
-void php_swoole_onBufferFull(swServer *, swDataHead *);
-void php_swoole_onBufferEmpty(swServer *, swDataHead *);
-ssize_t php_swoole_length_func(swProtocol *protocol, swSocket *_socket, char *data, uint32_t length);
-int php_swoole_client_onPackage(swConnection *conn, char *data, uint32_t length);
+ssize_t php_swoole_length_func(swProtocol *protocol, swSocket *_socket, const char *data, uint32_t length);
+int php_swoole_client_onPackage(swConnection *conn, const char *data, uint32_t length);
 zend_bool php_swoole_signal_isset_handler(int signo);
 
-#ifdef SW_USE_OPENSSL
-void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset);
-#endif
-
 ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool display_errors;
     zend_bool cli;
@@ -422,7 +360,6 @@ ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool enable_library;
     long socket_buffer_size;
     php_swoole_req_status req_status;
-    swLinkedList *rshutdown_functions;
 ZEND_END_MODULE_GLOBALS(swoole)
 
 extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
@@ -777,6 +714,9 @@ static sw_inline int add_assoc_ulong_safe(zval *arg, const char *key, zend_ulong
 #define SW_SET_CLASS_CREATE(module, _create_object) \
     module##_ce->create_object = _create_object
 
+#define SW_SET_CLASS_DTOR(module, _dtor_obj) \
+    module##_handlers.dtor_obj = _dtor_obj
+
 #define SW_SET_CLASS_FREE(module, _free_obj) \
     module##_handlers.free_obj = _free_obj
 
@@ -843,9 +783,22 @@ static sw_inline int sw_zend_register_class_alias(const char *name, size_t name_
 #endif
 }
 
+#if PHP_VERSION_ID < 70300
+/* Allocates object type and zeros it, but not the properties.
+ * Properties MUST be initialized using object_properties_init(). */
+static zend_always_inline void *zend_object_alloc(size_t obj_size, zend_class_entry *ce)
+{
+    void *obj = emalloc(obj_size + zend_object_properties_size(ce));
+    /* Subtraction of sizeof(zval) is necessary, because zend_object_properties_size() may be
+     * -sizeof(zval), if the object has no properties. */
+    memset(obj, 0, obj_size - sizeof(zval));
+    return obj;
+}
+#endif
+
 static sw_inline zend_object *sw_zend_create_object(zend_class_entry *ce, zend_object_handlers *handlers)
 {
-    zend_object* object = (zend_object *) ecalloc(1, sizeof(zend_object) + zend_object_properties_size(ce));
+    zend_object* object = (zend_object *) zend_object_alloc(sizeof(zend_object), ce);
     zend_object_std_init(object, ce);
     object_properties_init(object, ce);
     object->handlers = handlers;
@@ -957,9 +910,13 @@ static sw_inline zval* sw_zend_read_and_convert_property_array(zend_class_entry
 //----------------------------------Function API------------------------------------
 
 #if PHP_VERSION_ID < 80000
-#define SW_Z8_OBJ_P(zobj) zobj
+#define sw_zend7_object      zval
+#define SW_Z7_OBJ_P(object)  Z_OBJ_P(object)
+#define SW_Z8_OBJ_P(zobj)    zobj
 #else
-#define SW_Z8_OBJ_P(zobj) Z_OBJ_P(zobj)
+#define sw_zend7_object      zend_object
+#define SW_Z7_OBJ_P(object)  object
+#define SW_Z8_OBJ_P(zobj)    Z_OBJ_P(zobj)
 #endif
 
 /**
