@@ -14,9 +14,11 @@ using namespace std;
 using swoole::coroutine::System;
 using swoole::coroutine::Socket;
 
+double Socket::default_dns_timeout     = SW_DEFAULT_SOCKET_DNS_TIMEOUT;
 double Socket::default_connect_timeout = SW_DEFAULT_SOCKET_CONNECT_TIMEOUT;
 double Socket::default_read_timeout    = SW_DEFAULT_SOCKET_READ_TIMEOUT;
 double Socket::default_write_timeout   = SW_DEFAULT_SOCKET_WRITE_TIMEOUT;
+static thread_local char tmp_address[INET6_ADDRSTRLEN + 1];
 
 #ifdef SW_USE_OPENSSL
 #ifndef OPENSSL_NO_NEXTPROTONEG
@@ -153,11 +155,11 @@ bool Socket::add_event(const enum swEvent_type event)
     {
         if (socket->removed)
         {
-            ret = swoole_event_add(sock_fd, event, SW_FD_CORO_SOCKET) == SW_OK;
+            ret = swoole_event_add(socket, event) == SW_OK;
         }
         else
         {
-            ret = swoole_event_set(sock_fd, socket->events | event, SW_FD_CORO_SOCKET) == SW_OK;
+            ret = swoole_event_set(socket, socket->events | event) == SW_OK;
         }
     }
     set_err(ret ? 0 : errno);
@@ -207,7 +209,7 @@ bool Socket::wait_event(const enum swEvent_type event, const void **__buf, size_
     }
     else // if (event == SW_EVENT_WRITE)
     {
-        if (sw_unlikely(__n > 0 && *__buf != get_write_buffer()->str))
+        if (sw_unlikely(!zero_copy && __n > 0 && *__buf != get_write_buffer()->str))
         {
             swString_clear(write_buffer);
             if (swString_append_ptr(write_buffer, (const char *) *__buf, __n) != SW_OK)
@@ -230,11 +232,11 @@ bool Socket::wait_event(const enum swEvent_type event, const void **__buf, size_
         swReactor *reactor = SwooleTG.reactor;
         if (sw_likely(added_event == SW_EVENT_READ))
         {
-            swReactor_remove_read_event(reactor, sock_fd);
+            swReactor_remove_read_event(reactor, socket);
         }
         else // if (added_event == SW_EVENT_WRITE)
         {
-            swReactor_remove_write_event(reactor, sock_fd);
+            swReactor_remove_write_event(reactor, socket);
         }
     }
 #ifdef SW_USE_OPENSSL
@@ -410,6 +412,15 @@ bool Socket::http_proxy_handshake()
 
     //CONNECT
     int n;
+    const char *host = http_proxy->target_host;
+    int host_len = http_proxy->l_target_host;
+#ifdef SW_USE_OPENSSL
+    if (open_ssl && ssl_option.tls_host_name)
+    {
+        host = ssl_option.tls_host_name;
+        host_len = strlen(ssl_option.tls_host_name);
+    }
+#endif
     if (http_proxy->password)
     {
         char auth_buf[256];
@@ -422,9 +433,9 @@ bool Socket::http_proxy_handshake()
         swBase64_encode((unsigned char *) auth_buf, n, encode_buf);
         n = sw_snprintf(
             buffer->str, buffer->size,
-            HTTP_PROXY_FMT "Proxy-Authorization:Basic %s\r\n\r\n",
-            http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port,
+            HTTP_PROXY_FMT "Proxy-Authorization: Basic %s\r\n\r\n",
             http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port,
+            host_len, host, http_proxy->target_port,
             encode_buf
         );
     }
@@ -434,7 +445,7 @@ bool Socket::http_proxy_handshake()
             buffer->str, buffer->size,
             HTTP_PROXY_FMT "\r\n",
             http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port,
-            http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port
+            host_len, host, http_proxy->target_port
         );
     }
 
@@ -529,68 +540,39 @@ bool Socket::http_proxy_handshake()
 void Socket::init_sock_type(enum swSocket_type _sw_type)
 {
     type = _sw_type;
-    switch (type)
-    {
-    case SW_SOCK_TCP6:
-        sock_domain = AF_INET6;
-        sock_type = SOCK_STREAM;
-        break;
-    case SW_SOCK_UNIX_STREAM:
-        sock_domain = AF_UNIX;
-        sock_type = SOCK_STREAM;
-        break;
-    case SW_SOCK_UDP:
-        sock_domain = AF_INET;
-        sock_type = SOCK_DGRAM;
-        break;
-    case SW_SOCK_UDP6:
-        sock_domain = AF_INET6;
-        sock_type = SOCK_DGRAM;
-        break;
-    case SW_SOCK_UNIX_DGRAM:
-        sock_domain = AF_UNIX;
-        sock_type = SOCK_DGRAM;
-        break;
-    case SW_SOCK_TCP:
-    default:
-        sock_domain = AF_INET;
-        sock_type = SOCK_STREAM;
-        break;
-    }
+    swSocket_get_domain_and_type(_sw_type, &sock_domain, &sock_type);
 }
 
 bool Socket::init_sock()
 {
-#ifdef SOCK_CLOEXEC
-    int _fd = ::socket(sock_domain, sock_type | SOCK_CLOEXEC, sock_protocol);
-#else
-    int _fd = ::socket(sock_domain, sock_type, sock_protocol);
-#endif
+    int _fd = swSocket_create(type, 1, 1);
     if (sw_unlikely(_fd < 0))
     {
         return false;
     }
-    init_reactor_socket(_fd);
-    return true;
+    return init_reactor_socket(_fd);
 }
 
-void Socket::init_reactor_socket(int _fd)
+bool Socket::init_reactor_socket(int _fd)
 {
     swReactor *reactor = SwooleTG.reactor;
     if (sw_unlikely(!reactor))
     {
         swFatalError(SW_ERROR_OPERATION_NOT_SUPPORT, "operation not support (reactor is not ready)");
     }
+    socket = swSocket_new(_fd, SW_FD_CORO_SOCKET);
+    if (socket == nullptr)
+    {
+        return false;
+    }
 
-    socket = swReactor_get(reactor, _fd);
-    bzero(socket, sizeof(swSocket));
-    sock_fd = socket->fd = _fd;
+    sock_fd = _fd;
     socket->object = this;
     socket->socket_type = type;
-    socket->removed = 1;
-    socket->fdtype = SW_FD_CORO_SOCKET;
+    socket->nonblock = 1;
+    socket->cloexec = 1;
 
-    swSocket_set_nonblock(sock_fd);
+    return true;
 }
 
 Socket::Socket(int _domain, int _type, int _protocol) :
@@ -617,7 +599,11 @@ Socket::Socket(enum swSocket_type _type)
 Socket::Socket(int _fd, enum swSocket_type _type)
 {
     init_sock_type(_type);
-    init_reactor_socket(_fd);
+    if (sw_unlikely(!init_reactor_socket(_fd)))
+    {
+        return;
+    }
+    swSocket_set_nonblock(socket);
     init_options();
 }
 
@@ -625,19 +611,28 @@ Socket::Socket(int _fd, int _domain, int _type, int _protocol) :
         sock_domain(_domain), sock_type(_type), sock_protocol(_protocol)
 {
     type = convert_to_type(_domain, _type, _protocol);
-    init_reactor_socket(_fd);
+    if (sw_unlikely(!init_reactor_socket(_fd)))
+    {
+        return;
+    }
+    swSocket_set_nonblock(socket);
     init_options();
 }
 
-Socket::Socket(int _fd, swSocketAddress *addr, Socket *server_sock)
+Socket::Socket(swSocket *sock, Socket *server_sock)
 {
     type = server_sock->type;
     sock_domain = server_sock->sock_domain;
     sock_type = server_sock->sock_type;
     sock_protocol = server_sock->sock_protocol;
-    init_reactor_socket(_fd);
+    sock_fd = sock->fd;
+    socket = sock;
+    socket->object = this;
+    socket->socket_type = type;
+    socket->fdtype = SW_FD_CORO_SOCKET;
     init_options();
     /* inherits server socket options */
+    dns_timeout = server_sock->dns_timeout;
     connect_timeout = server_sock->connect_timeout;
     read_timeout = server_sock->read_timeout;
     write_timeout = server_sock->write_timeout;
@@ -645,18 +640,17 @@ Socket::Socket(int _fd, swSocketAddress *addr, Socket *server_sock)
     open_eof_check = server_sock->open_eof_check;
     http2 = server_sock->http2;
     protocol = server_sock->protocol;
-
-    socket->socket_type = server_sock->socket->socket_type;
-    memcpy(&socket->info.addr, addr, addr->len);
-    socket->info.len = addr->len;
-
 #ifdef SW_USE_OPENSSL
-    if (server_sock->open_ssl)
+    open_ssl = server_sock->open_ssl;
+    ssl_is_server = server_sock->ssl_is_server;
+    if (open_ssl)
     {
-        if (swSSL_create(socket, server_sock->ssl_context, 0) < 0 || !ssl_accept())
+        if (server_sock->ssl_context)
         {
-            close();
-            return;
+            if (!ssl_create(server_sock->ssl_context))
+            {
+                close();
+            }
         }
     }
 #endif
@@ -688,11 +682,13 @@ const char* Socket::get_ip()
 {
     if (type == SW_SOCK_TCP || type == SW_SOCK_UDP)
     {
-        return inet_ntoa(socket->info.addr.inet_v4.sin_addr);
+        if (inet_ntop(AF_INET, &socket->info.addr.inet_v4.sin_addr, tmp_address, sizeof(tmp_address)))
+        {
+            return tmp_address;
+        }
     }
     else if (type == SW_SOCK_TCP6 || type == SW_SOCK_UDP6)
     {
-        static char tmp_address[INET6_ADDRSTRLEN + 1];
         if (inet_ntop(AF_INET6, &socket->info.addr.inet_v6.sin6_addr, tmp_address, sizeof(tmp_address)))
         {
             return tmp_address;
@@ -789,7 +785,6 @@ bool Socket::connect(string _host, int _port, int flags)
 #endif
     if (socks5_proxy)
     {
-        //enable socks5 proxy
         socks5_proxy->target_host = sw_strndup((char *) _host.c_str(), _host.size());
         socks5_proxy->l_target_host = _host.size();
         socks5_proxy->target_port = _port;
@@ -799,7 +794,6 @@ bool Socket::connect(string _host, int _port, int flags)
     }
     else if (http_proxy)
     {
-        //enable http proxy
         http_proxy->target_host = sw_strndup((char *) _host.c_str(), _host.size());
         http_proxy->l_target_host = _host.size();
         http_proxy->target_port = _port;
@@ -817,7 +811,7 @@ bool Socket::connect(string _host, int _port, int flags)
         }
         else if (_port == 0 || _port >= 65536)
         {
-            set_err(EINVAL, cpp_string::format("Invalid port [%d]", _port).c_str());
+            set_err(EINVAL, cpp_string::format("Invalid port [%d]", _port));
             return false;
         }
     }
@@ -837,18 +831,18 @@ bool Socket::connect(string _host, int _port, int flags)
             if (!inet_pton(AF_INET, connect_host.c_str(), &socket->info.addr.inet_v4.sin_addr))
             {
 #ifdef SW_USE_OPENSSL
-                if (open_ssl)
+                if (open_ssl && !(socks5_proxy || http_proxy))
                 {
                     ssl_host_name = connect_host;
                 }
 #endif
                 /* locked like wait_event */
                 read_co = write_co = Coroutine::get_current_safe();
-                connect_host = System::gethostbyname(connect_host, AF_INET, connect_timeout);
+                connect_host = System::gethostbyname(connect_host, AF_INET, dns_timeout);
                 read_co = write_co = nullptr;
                 if (connect_host.empty())
                 {
-                    set_err(SwooleG.error, swoole_strerror(SwooleG.error));
+                    set_err(swoole_get_last_error(), swoole_strerror(swoole_get_last_error()));
                     return false;
                 }
                 continue;
@@ -868,15 +862,15 @@ bool Socket::connect(string _host, int _port, int flags)
             if (!inet_pton(AF_INET6, connect_host.c_str(), &socket->info.addr.inet_v6.sin6_addr))
             {
 #ifdef SW_USE_OPENSSL
-                if (open_ssl)
+                if (open_ssl && !(socks5_proxy || http_proxy))
                 {
                     ssl_host_name = connect_host;
                 }
 #endif
-                connect_host = System::gethostbyname(connect_host, AF_INET6, connect_timeout);
+                connect_host = System::gethostbyname(connect_host, AF_INET6, dns_timeout);
                 if (connect_host.empty())
                 {
-                    set_err(SwooleG.error);
+                    set_err(swoole_get_last_error());
                     return false;
                 }
                 continue;
@@ -920,9 +914,13 @@ bool Socket::connect(string _host, int _port, int flags)
         return false;
     }
 #ifdef SW_USE_OPENSSL
-    if (open_ssl && ssl_handshake() == false)
+    ssl_is_server = false;
+    if (open_ssl)
     {
-        return false;
+        if (!ssl_handshake())
+        {
+            return false;
+        }
     }
 #endif
     return true;
@@ -937,10 +935,10 @@ bool Socket::check_liveness()
     }
     else
     {
-        static char buf;
+        char buf;
         errno = 0;
-        int ret = swConnection_peek(socket, &buf, sizeof(buf), 0);
-        if (ret == 0 || (ret < 0 && swConnection_error(errno) != SW_WAIT)) {
+        ssize_t retval = swSocket_peek(socket, &buf, sizeof(buf), 0);
+        if (retval == 0 || (retval < 0 && swSocket_error(errno) != SW_WAIT)) {
             set_err(errno ? errno : ECONNRESET);
             return false;
         }
@@ -951,7 +949,7 @@ bool Socket::check_liveness()
 
 ssize_t Socket::peek(void *__buf, size_t __n)
 {
-    ssize_t retval = swConnection_peek(socket, __buf, __n, 0);
+    ssize_t retval = swSocket_peek(socket, __buf, __n, 0);
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -983,8 +981,8 @@ ssize_t Socket::recv(void *__buf, size_t __n)
     timer_controller timer(&read_timer, read_timeout, this, timer_callback);
     do
     {
-        retval = swConnection_recv(socket, __buf, __n, 0);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
+        retval = swSocket_recv(socket, __buf, __n, 0);
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -998,8 +996,8 @@ ssize_t Socket::send(const void *__buf, size_t __n)
     ssize_t retval;
     timer_controller timer(&write_timer, write_timeout, this, timer_callback);
     do {
-        retval = swConnection_send(socket, __buf, __n, 0);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
+        retval = swSocket_send(socket, __buf, __n, 0);
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1014,7 +1012,7 @@ ssize_t Socket::read(void *__buf, size_t __n)
     timer_controller timer(&read_timer, read_timeout, this, timer_callback);
     do {
         retval = ::read(sock_fd, __buf, __n);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1029,7 +1027,7 @@ ssize_t Socket::write(const void *__buf, size_t __n)
     timer_controller timer(&write_timer, write_timeout, this, timer_callback);
     do {
         retval = ::write(sock_fd, (void *) __buf, __n);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1045,8 +1043,8 @@ ssize_t Socket::recv_all(void *__buf, size_t __n)
     while (true)
     {
         do {
-            retval = swConnection_recv(socket, (char *) __buf + total_bytes, __n - total_bytes, 0);
-        } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
+            retval = swSocket_recv(socket, (char *) __buf + total_bytes, __n - total_bytes, 0);
+        } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
         if (sw_unlikely(retval <= 0))
         {
             if (total_bytes == 0)
@@ -1077,9 +1075,9 @@ ssize_t Socket::send_all(const void *__buf, size_t __n)
     {
         do
         {
-            retval = swConnection_send(socket, (char *) __buf + total_bytes, __n - total_bytes, 0);
+            retval = swSocket_send(socket, (char *) __buf + total_bytes, __n - total_bytes, 0);
         }
-        while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
+        while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n));
         /**
          * failed to send
          */
@@ -1111,7 +1109,7 @@ ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
     timer_controller timer(&read_timer, read_timeout, this, timer_callback);
     do {
         retval = ::recvmsg(sock_fd, msg, flags);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1129,7 +1127,7 @@ ssize_t Socket::sendmsg(const struct msghdr *msg, int flags)
     timer_controller timer(&write_timer, write_timeout, this, timer_callback);
     do {
         retval = ::sendmsg(sock_fd, msg, flags);
-    } while (retval < 0 && swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE));
+    } while (retval < 0 && swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1142,7 +1140,7 @@ bool Socket::bind(std::string address, int port)
     }
     if ((sock_domain == AF_INET || sock_domain == AF_INET6) && (port < 0 || port > 65535))
     {
-        set_err(EINVAL, cpp_string::format("Invalid port [%d]", port).c_str());
+        set_err(EINVAL, cpp_string::format("Invalid port [%d]", port));
         return false;
     }
 
@@ -1156,16 +1154,6 @@ bool Socket::bind(std::string address, int port)
     {
         swSysWarn("setsockopt(%d, SO_REUSEADDR) failed", sock_fd);
     }
-#ifdef HAVE_REUSEPORT
-    if (SwooleG.reuse_port)
-    {
-        if (::setsockopt(sock_fd, SOL_SOCKET, SO_REUSEPORT, &option, sizeof(int)) < 0)
-        {
-            swSysWarn("setsockopt(SO_REUSEPORT) failed");
-            SwooleG.reuse_port = 0;
-        }
-    }
-#endif
 
     int retval;
     socklen_t len;
@@ -1183,7 +1171,7 @@ bool Socket::bind(std::string address, int port)
                 cpp_string::format(
                     "UNIXSocket bind path(%s) is too long, the maxium limit of bytes number is %zu",
                     bind_address.c_str(), sizeof(sa->sun_path)
-                ).c_str()
+                )
             );
             return false;
         }
@@ -1264,6 +1252,7 @@ bool Socket::listen(int backlog)
         return false;
     }
 #ifdef SW_USE_OPENSSL
+    ssl_is_server = true;
     if (open_ssl)
     {
         return ssl_check_context();
@@ -1279,22 +1268,23 @@ Socket* Socket::accept(double timeout)
         return nullptr;
     }
     swSocketAddress client_addr;
-    int conn = swSocket_accept(sock_fd, &client_addr);
-    if (conn < 0 && errno == EAGAIN)
+    swSocket *conn = swSocket_accept(socket, &client_addr);
+    if (conn == nullptr && errno == EAGAIN)
     {
         timer_controller timer(&read_timer, timeout == 0 ? read_timeout : timeout, this, timer_callback);
         if (!timer.start() || !wait_event(SW_EVENT_READ))
         {
             return nullptr;
         }
-        conn = swSocket_accept(sock_fd, &client_addr);
+        conn = swSocket_accept(socket, &client_addr);
     }
-    if (conn < 0)
+    if (conn == nullptr)
     {
         set_err(errno);
         return nullptr;
     }
-    Socket *client_sock = new Socket(conn, &client_addr, this);
+
+    Socket *client_sock = new Socket(conn, this);
     if (sw_unlikely(client_sock->get_fd() < 0))
     {
         swSysWarn("new Socket() failed");
@@ -1302,44 +1292,34 @@ Socket* Socket::accept(double timeout)
         delete client_sock;
         return nullptr;
     }
+
     return client_sock;
 }
 
 #ifdef SW_USE_OPENSSL
 bool Socket::ssl_check_context()
 {
-    if (ssl_context)
+    if (socket->ssl || ssl_context)
     {
         return true;
     }
+    if (swSocket_is_dgram(sock_type))
+    {
+#ifdef SW_SUPPORT_DTLS
+        socket->dtls = 1;
+        ssl_option.dtls = 1;
+        ssl_option.method = SW_DTLS_CLIENT_METHOD;
+#else
+        swWarn("DTLS support require openssl-1.1 or later");
+        return false;
+#endif
+    }
     ssl_context = swSSL_get_context(&ssl_option);
     if (ssl_context == nullptr)
     {
         swWarn("swSSL_get_context() error");
         return false;
     }
-    else
-    {
-        return true;
-    }
-}
-
-bool Socket::ssl_handshake()
-{
-    if (sw_unlikely(!is_available(SW_EVENT_RDWR)))
-    {
-        return -1;
-    }
-    if (socket->ssl)
-    {
-        return false;
-    }
-    ssl_context = swSSL_get_context(&ssl_option);
-    if (ssl_context == NULL)
-    {
-        return false;
-    }
-
     if (ssl_option.verify_peer)
     {
         if (swSSL_set_capath(&ssl_option, ssl_context) < 0)
@@ -1347,7 +1327,6 @@ bool Socket::ssl_handshake()
             return false;
         }
     }
-
     socket->ssl_send = 1;
 #if defined(SW_USE_HTTP2) && defined(SW_USE_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10002000L
     if (http2)
@@ -1361,8 +1340,16 @@ bool Socket::ssl_handshake()
         }
     }
 #endif
+    return true;
+}
 
-    if (swSSL_create(socket, ssl_context, SW_SSL_CLIENT) < 0)
+bool Socket::ssl_create(SSL_CTX *ssl_context)
+{
+    if (socket->ssl)
+    {
+        return true;
+    }
+    if (swSSL_create(socket, ssl_context, 0) < 0)
     {
         return false;
     }
@@ -1379,71 +1366,110 @@ bool Socket::ssl_handshake()
         SSL_set_tlsext_host_name(socket->ssl, ssl_host_name.c_str());
     }
 #endif
-
-    while (true)
-    {
-        if (swSSL_connect(socket) < 0)
-        {
-            set_err(errno);
-            return false;
-        }
-        if (socket->ssl_state == SW_SSL_STATE_WAIT_STREAM)
-        {
-            timer_controller timer(&read_timer, read_timeout, this, timer_callback);
-            if (!timer.start() || !wait_event(SW_EVENT_READ))
-            {
-                return false;
-            }
-        }
-        else if (socket->ssl_state == SW_SSL_STATE_READY)
-        {
-            return true;
-        }
-    }
-
-    if (socket->ssl_state == SW_SSL_STATE_READY && ssl_option.verify_peer)
-    {
-        if (ssl_verify(ssl_option.allow_self_signed) < 0)
-        {
-            return false;
-        }
-    }
     return true;
 }
 
-bool Socket::ssl_accept()
+bool Socket::ssl_handshake()
 {
-    int retval;
-    timer_controller timer(&read_timer, read_timeout, this, timer_callback);
-    open_ssl = true;
-    do {
-        retval = swSSL_accept(socket);
-    } while (retval == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
-
-    if (retval != SW_READY)
+    if (ssl_handshaked)
     {
-        set_err(SW_ERROR_SSL_BAD_CLIENT);
         return false;
     }
+    if (sw_unlikely(!is_available(SW_EVENT_RDWR)))
+    {
+        return false;
+    }
+    if (!ssl_check_context())
+    {
+        return false;
+    }
+    if (!ssl_create(ssl_context))
+    {
+        return false;
+    }
+    if (!ssl_is_server) {
+        while (true)
+        {
+            if (swSSL_connect(socket) < 0)
+            {
+                set_err(errno);
+                return false;
+            }
+            if (socket->ssl_state == SW_SSL_STATE_WAIT_STREAM)
+            {
+                timer_controller timer(&read_timer, read_timeout, this, timer_callback);
+                if (!timer.start() || !wait_event(SW_EVENT_READ))
+                {
+                    return false;
+                }
+            }
+            else if (socket->ssl_state == SW_SSL_STATE_READY)
+            {
+                break;
+            }
+        }
+    }
     else
     {
-        return true;
+        enum swReturn_code retval;
+        timer_controller timer(&read_timer, read_timeout, this, timer_callback);
+
+        do
+        {
+            retval = swSSL_accept(socket);
+        }
+        while (retval == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ));
+
+        if (retval != SW_READY)
+        {
+            set_err(SW_ERROR_SSL_BAD_CLIENT);
+            return false;
+        }
     }
+    if (ssl_option.verify_peer)
+    {
+        if (!ssl_verify(ssl_option.allow_self_signed))
+        {
+            return false;
+        }
+    }
+    open_ssl = true;
+    ssl_handshaked = true;
+
+    return true;
 }
 
-int Socket::ssl_verify(bool allow_self_signed)
+bool Socket::ssl_verify(bool allow_self_signed)
 {
     if (swSSL_verify(socket, allow_self_signed) < 0)
     {
-        return SW_ERR;
+        set_err(SW_ERROR_SSL_VERIFY_FAILED);
+        return false;
     }
 #ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
     if (ssl_option.tls_host_name && swSSL_check_host(socket, ssl_option.tls_host_name) < 0)
     {
-        return SW_ERR;
+        return false;
     }
 #endif
-    return SW_OK;
+    return true;
+}
+
+std::string Socket::ssl_get_peer_cert()
+{
+    if (!socket->ssl)
+    {
+        return "";
+    }
+    int n = swSSL_get_peer_cert(socket->ssl, SwooleTG.buffer_stack->str, SwooleTG.buffer_stack->size);
+    if (n <= 0)
+    {
+        return "";
+    }
+    else
+    {
+        return std::string(SwooleTG.buffer_stack->str, n);
+    }
 }
 #endif
 
@@ -1456,7 +1482,7 @@ bool Socket::sendfile(const char *filename, off_t offset, size_t length)
     int file_fd = ::open(filename, O_RDONLY);
     if (file_fd < 0)
     {
-        set_err(errno, cpp_string::format("open(%s) failed, %s", filename, strerror(errno)).c_str());
+        set_err(errno, cpp_string::format("open(%s) failed, %s", filename, strerror(errno)));
         return false;
     }
 
@@ -1465,7 +1491,7 @@ bool Socket::sendfile(const char *filename, off_t offset, size_t length)
         struct stat file_stat;
         if (::fstat(file_fd, &file_stat) < 0)
         {
-            set_err(errno, cpp_string::format("fstat(%s) failed, %s", filename, strerror(errno)).c_str());
+            set_err(errno, cpp_string::format("fstat(%s) failed, %s", filename, strerror(errno)));
             ::close(file_fd);
             return false;
         }
@@ -1504,7 +1530,7 @@ bool Socket::sendfile(const char *filename, off_t offset, size_t length)
         }
         else if (errno != EAGAIN)
         {
-            set_err(errno, cpp_string::format("sendfile(%d, %s) failed, %s", sock_fd, filename, strerror(errno)).c_str());
+            set_err(errno, cpp_string::format("sendfile(%d, %s) failed, %s", sock_fd, filename, strerror(errno)));
             ::close(file_fd);
             return false;
         }
@@ -1518,7 +1544,7 @@ bool Socket::sendfile(const char *filename, off_t offset, size_t length)
     return true;
 }
 
-ssize_t Socket::sendto(const char *address, int port, const void *__buf, size_t __n)
+ssize_t Socket::sendto(const string &host, int port, const void *__buf, size_t __n)
 {
     if (sw_unlikely(!is_available(SW_EVENT_WRITE)))
     {
@@ -1534,45 +1560,67 @@ ssize_t Socket::sendto(const char *address, int port, const void *__buf, size_t
     } addr = {};
     size_t addr_size = 0;
 
-    switch (type)
+    std::string ip = host;
+
+    for (size_t i = 0; i < 2; i++)
     {
-    case SW_SOCK_UDP:
-    {
-        if (::inet_aton(address, &addr.in.sin_addr) == 0)
+        if (type == SW_SOCK_UDP)
         {
-            set_err(EINVAL, cpp_string::format("ip[%s] is invalid", address).c_str());
+            if (::inet_aton(ip.c_str(), &addr.in.sin_addr) == 0)
+            {
+                read_co = write_co = Coroutine::get_current_safe();
+                ip = System::gethostbyname(host, sock_domain, dns_timeout);
+                read_co = write_co = nullptr;
+                if (ip.empty())
+                {
+                    set_err(swoole_get_last_error(), swoole_strerror(swoole_get_last_error()));
+                    return -1;
+                }
+                continue;
+            }
+            else
+            {
+                addr.in.sin_family = AF_INET;
+                addr.in.sin_port = htons(port);
+                addr_size = sizeof(addr.in);
+                break;
+            }
+        }
+        else if (type == SW_SOCK_UDP6)
+        {
+            if (::inet_pton(AF_INET6, ip.c_str(), &addr.in6.sin6_addr) == 0)
+            {
+                read_co = write_co = Coroutine::get_current_safe();
+                ip = System::gethostbyname(host, sock_domain, dns_timeout);
+                read_co = write_co = nullptr;
+                if (ip.empty())
+                {
+                    set_err(swoole_get_last_error(), swoole_strerror(swoole_get_last_error()));
+                    return -1;
+                }
+                continue;
+            }
+            else
+            {
+                addr.in6.sin6_port = (uint16_t) htons(port);
+                addr.in6.sin6_family = AF_INET6;
+                addr_size = sizeof(addr.in6);
+                break;
+            }
+        }
+        else if (type == SW_SOCK_UNIX_DGRAM)
+        {
+            addr.un.sun_family = AF_UNIX;
+            strncpy(addr.un.sun_path, host.c_str(), sizeof(addr.un.sun_path) - 1);
+            addr_size = sizeof(addr.un);
+            break;
+        }
+        else
+        {
+            set_err(EPROTONOSUPPORT);
             retval = -1;
             break;
         }
-        addr.in.sin_family = AF_INET;
-        addr.in.sin_port = htons(port);
-        addr_size = sizeof(addr.in);
-        break;
-    }
-    case SW_SOCK_UDP6:
-    {
-        if (::inet_pton(AF_INET6, address, &addr.in6.sin6_addr) < 0)
-        {
-            set_err(EINVAL, cpp_string::format("ip[%s] is invalid", address).c_str());
-            retval = -1;
-            break;
-        }
-        addr.in6.sin6_port = (uint16_t) htons(port);
-        addr.in6.sin6_family = AF_INET6;
-        addr_size = sizeof(addr.in6);
-        break;
-    }
-    case SW_SOCK_UNIX_DGRAM:
-    {
-        addr.un.sun_family = AF_UNIX;
-        strncpy(addr.un.sun_path, address, sizeof(addr.un.sun_path) - 1);
-        addr_size = sizeof(addr.un);
-        break;
-    }
-    default:
-        set_err(EPROTONOSUPPORT);
-        retval = -1;
-        break;
     }
 
     if (addr_size > 0)
@@ -1581,7 +1629,7 @@ ssize_t Socket::sendto(const char *address, int port, const void *__buf, size_t
         do {
             retval = ::sendto(sock_fd, __buf, __n, 0, (struct sockaddr *) &addr, addr_size);
             swTraceLog(SW_TRACE_SOCKET, "sendto %ld/%ld bytes, errno=%d", retval, __n, errno);
-        } while (retval < 0 && (errno == EINTR || (swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n))));
+        } while (retval < 0 && (errno == EINTR || (swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_WRITE, &__buf, __n))));
         set_err(retval < 0 ? errno : 0);
     }
 
@@ -1609,7 +1657,7 @@ ssize_t Socket::recvfrom(void *__buf, size_t __n, struct sockaddr* _addr, sockle
     do {
         retval = ::recvfrom(sock_fd, __buf, __n, 0, _addr, _socklen);
         swTraceLog(SW_TRACE_SOCKET, "recvfrom %ld/%ld bytes, errno=%d", retval, __n, errno);
-    } while (retval < 0 && ((errno == EINTR) || (swConnection_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ))));
+    } while (retval < 0 && ((errno == EINTR) || (swSocket_error(errno) == SW_WAIT && timer.start() && wait_event(SW_EVENT_READ))));
     set_err(retval < 0 ? errno : 0);
     return retval;
 }
@@ -1952,17 +2000,6 @@ bool Socket::close()
     }
 }
 
-static void socket_close_fd(void *ptr)
-{
-    swSocket *sock = (swSocket *) ptr;
-    if (::close(sock->fd) != 0)
-    {
-        swSysWarn("close(%d) failed", sock->fd);
-    }
-    bzero(sock, sizeof(swSocket));
-    sock->removed = 1;
-}
-
 /**
  * Warn:
  * the destructor should only be called in following two cases:
@@ -2038,13 +2075,5 @@ Socket::~Socket()
     {
         ::unlink(socket->info.addr.un.sun_path);
     }
-    if (SwooleTG.reactor)
-    {
-        socket->removed = 1;
-        swoole_event_defer(socket_close_fd, socket);
-    }
-    else
-    {
-        socket_close_fd(socket);
-    }
+    swSocket_free(socket);
 }
