@@ -142,7 +142,7 @@ swString* System::read_file(const char *file, bool lock)
     }
     else
     {
-        SwooleG.error = ev.error;
+        swoole_set_last_error(ev.error);
         return NULL;
     }
 }
@@ -174,7 +174,7 @@ ssize_t System::write_file(const char *file, char *buf, size_t length, bool lock
     task.co->yield();
     if (ev.error != 0)
     {
-        SwooleG.error = ev.error;
+        swoole_set_last_error(ev.error);
     }
     return ev.ret;
 }
@@ -252,7 +252,7 @@ string System::gethostbyname(const string &hostname, int domain, double timeout)
 
     if (ev.ret == -1)
     {
-        SwooleG.error = ev.error;
+        swoole_set_last_error(ev.error);
         return "";
     }
     else
@@ -318,7 +318,7 @@ vector<string> System::getaddrinfo(const string &hostname, int family, int sockt
 
     if (ev.ret == -1)
     {
-        SwooleG.error = ev.error;
+        swoole_set_last_error(ev.error);
     }
 
     struct sockaddr_in *addr_v4;
@@ -350,7 +350,7 @@ vector<string> System::getaddrinfo(const string &hostname, int family, int sockt
     }
     else
     {
-        SwooleG.error = ev.error;
+        swoole_set_last_error(ev.error);
     }
 
     return retval;
@@ -365,16 +365,24 @@ struct coro_poll_task
     bool wait = true;
 };
 
-static std::unordered_map<int, coro_poll_task *> coro_poll_task_map;
-
 static inline void socket_poll_clean(coro_poll_task *task)
 {
     for (auto i = task->fds->begin(); i != task->fds->end(); i++)
     {
-        coro_poll_task_map.erase(i->first);
-        if (swoole_event_del(i->first) < 0)
+        swSocket *socket = i->second.socket;
+        if (!socket)
+        {
+            continue;
+        }
+        int retval = swoole_event_del(i->second.socket);
+        /**
+         * Temporary socket, fd marked -1, skip close
+         */
+        socket->fd = -1;
+        swSocket_free(socket);
+        i->second.socket = nullptr;
+        if (retval < 0)
         {
-            //TODO print error log
             continue;
         }
     }
@@ -397,9 +405,8 @@ static void socket_poll_completed(void *data)
     task->co->resume();
 }
 
-static inline void socket_poll_trigger_event(swReactor *reactor, int fd, enum swEvent_type event)
+static inline void socket_poll_trigger_event(swReactor *reactor, coro_poll_task *task, int fd, enum swEvent_type event)
 {
-    coro_poll_task *task = coro_poll_task_map[fd];
     auto i = task->fds->find(fd);
     if (event == SW_EVENT_ERROR && !(i->second.events & SW_EVENT_ERROR))
     {
@@ -431,28 +438,57 @@ static inline void socket_poll_trigger_event(swReactor *reactor, int fd, enum sw
 
 static int socket_poll_read_callback(swReactor *reactor, swEvent *event)
 {
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_READ);
+    socket_poll_trigger_event(reactor, (coro_poll_task *) event->socket->object, event->fd, SW_EVENT_READ);
     return SW_OK;
 }
 
 static int socket_poll_write_callback(swReactor *reactor, swEvent *event)
 {
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_WRITE);
+    socket_poll_trigger_event(reactor, (coro_poll_task *) event->socket->object, event->fd, SW_EVENT_WRITE);
     return SW_OK;
 }
 
 static int socket_poll_error_callback(swReactor *reactor, swEvent *event)
 {
-    socket_poll_trigger_event(reactor, event->fd, SW_EVENT_ERROR);
+    socket_poll_trigger_event(reactor, (coro_poll_task *) event->socket->object, event->fd, SW_EVENT_ERROR);
     return SW_OK;
 }
 
-void System::init_reactor(swReactor *reactor)
+static int translate_events_to_poll(int events)
 {
-    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_READ, socket_poll_read_callback);
-    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_WRITE, socket_poll_write_callback);
-    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_ERROR, socket_poll_error_callback);
-    swReactor_set_handler(reactor, SW_FD_AIO | SW_EVENT_READ, swAio_callback);
+    int poll_events = 0;
+
+    if (events & SW_EVENT_READ)
+    {
+        poll_events |= POLLIN;
+    }
+    if (events & SW_EVENT_WRITE)
+    {
+        poll_events |= POLLOUT;
+    }
+
+    return poll_events;
+}
+
+static int translate_events_from_poll(int events)
+{
+    int sw_events = 0;
+
+    if (events & POLLIN)
+    {
+        sw_events |= SW_EVENT_READ;
+    }
+    if (events & POLLOUT)
+    {
+        sw_events |= SW_EVENT_WRITE;
+    }
+    //ignore ERR and HUP, because event is already processed at IN and OUT handler.
+    if ((((events & POLLERR) || (events & POLLHUP)) && !((events & POLLIN) || (events & POLLOUT))))
+    {
+        sw_events |= SW_EVENT_ERROR;
+    }
+
+    return sw_events;
 }
 
 bool System::socket_poll(std::unordered_map<int, socket_poll_fd> &fds, double timeout)
@@ -462,39 +498,23 @@ bool System::socket_poll(std::unordered_map<int, socket_poll_fd> &fds, double ti
         struct pollfd *event_list = (struct pollfd *) sw_calloc(fds.size(), sizeof(struct pollfd));
         if (!event_list)
         {
-            swWarn("malloc[1] failed");
+            swWarn("calloc() failed");
             return false;
         }
-        int j = 0;
-        for (auto i = fds.begin(); i != fds.end(); i++)
+        int n = 0;
+        for (auto i = fds.begin(); i != fds.end(); i++, n++)
         {
-            event_list[j].fd = i->first;
-            event_list[j].events = i->second.events;
-            event_list[j].revents = 0;
-            j++;
+            event_list[n].fd = i->first;
+            event_list[n].events = translate_events_to_poll(i->second.events);
+            event_list[n].revents = 0;
         }
-        int retval = ::poll(event_list, fds.size(), 0);
+        int retval = ::poll(event_list, n, 0);
         if (retval > 0)
         {
-            for (size_t i = 0; i < fds.size(); i++)
+            int n = 0;
+            for (auto i = fds.begin(); i != fds.end(); i++, n++)
             {
-                auto _e = fds.find(event_list[i].fd);
-                int16_t revents = event_list[i].revents;
-                int16_t sw_revents = 0;
-                if (revents & POLLIN)
-                {
-                    sw_revents |= SW_EVENT_READ;
-                }
-                if (revents & POLLOUT)
-                {
-                    sw_revents |= SW_EVENT_WRITE;
-                }
-                //ignore ERR and HUP, because event is already processed at IN and OUT handler.
-                if ((((revents & POLLERR) || (revents & POLLHUP)) && !((revents & POLLIN) || (revents & POLLOUT))))
-                {
-                    sw_revents |= SW_EVENT_ERROR;
-                }
-                _e->second.revents = sw_revents;
+                i->second.revents = translate_events_from_poll(event_list[n].revents);
             }
         }
         sw_free(event_list);
@@ -508,15 +528,17 @@ bool System::socket_poll(std::unordered_map<int, socket_poll_fd> &fds, double ti
 
     for (auto i = fds.begin(); i != fds.end(); i++)
     {
-        if (swoole_event_add(i->first, i->second.events, SW_FD_CORO_POLL) < 0)
+        i->second.socket = swSocket_new(i->first, SW_FD_CORO_POLL);
+        if (i->second.socket == nullptr)
         {
             continue;
         }
-        else
+        if (swoole_event_add(i->second.socket, i->second.events) < 0)
         {
-            coro_poll_task_map[i->first] = &task;
-            tasked_num++;
+            continue;
         }
+        i->second.socket->object = &task;
+        tasked_num++;
     }
 
     if (sw_unlikely(tasked_num == 0))
@@ -534,6 +556,147 @@ bool System::socket_poll(std::unordered_map<int, socket_poll_fd> &fds, double ti
     return task.success;
 }
 
+struct event_waiter
+{
+    swSocket *socket;
+    swTimer_node *timer;
+    Coroutine *co;
+    int revents;
+
+    event_waiter(int fd, int events, double timeout)
+    {
+        revents = 0;
+        if (!(socket = swSocket_new(fd, SW_FD_CORO_EVENT)))
+        {
+            swoole_set_last_error(errno);
+            return;
+        }
+        socket->object = this;
+        if (swoole_event_add(socket, events) < 0)
+        {
+            swoole_set_last_error(errno);
+            goto _done;
+        }
+        if (timeout > 0)
+        {
+            timer = swoole_timer_add((long) (timeout * 1000), SW_FALSE, [](swTimer *timer, swTimer_node *tnode){
+                event_waiter *waiter = (event_waiter *) tnode->data;
+                waiter->timer = nullptr;
+                waiter->co->resume();
+            }, this);
+        }
+        else
+        {
+            timer = nullptr;
+        }
+        co = Coroutine::get_current();
+
+        co->yield();
+
+        if (timer != nullptr)
+        {
+            swoole_timer_del(timer);
+        }
+        else if (timeout > 0)
+        {
+            swoole_set_last_error(ETIMEDOUT);
+        }
+        swoole_event_del(socket);
+        _done:
+        socket->fd = -1; /* skip close */
+        swSocket_free(socket);
+    }
+
+};
+
+static inline void event_waiter_callback(swReactor *reactor, event_waiter *waiter, enum swEvent_type event)
+{
+    if (waiter->revents == 0) {
+        reactor->defer(reactor, [](void *data) {
+            event_waiter *waiter = (event_waiter *) data;
+            waiter->co->resume();
+        }, waiter);
+    }
+    waiter->revents |= event;
+}
+
+static int event_waiter_read_callback(swReactor *reactor, swEvent *event)
+{
+    event_waiter_callback(reactor, (event_waiter *) event->socket->object, SW_EVENT_READ);
+    return SW_OK;
+}
+
+static int event_waiter_write_callback(swReactor *reactor, swEvent *event)
+{
+    event_waiter_callback(reactor, (event_waiter *) event->socket->object, SW_EVENT_WRITE);
+    return SW_OK;
+}
+
+static int event_waiter_error_callback(swReactor *reactor, swEvent *event)
+{
+    event_waiter_callback(reactor, (event_waiter *) event->socket->object, SW_EVENT_ERROR);
+    return SW_OK;
+}
+
+int System::wait_event(int fd, int events, double timeout)
+{
+    events &= SW_EVENT_READ | SW_EVENT_WRITE;
+    if (events == 0)
+    {
+        swoole_set_last_error(EINVAL);
+        return 0;
+    }
+
+    if (timeout == 0)
+    {
+        struct pollfd pfd;
+        pfd.fd = fd;
+        pfd.events = translate_events_to_poll(events);
+        pfd.revents = 0;
+
+        int retval = ::poll(&pfd, 1, 0);
+        if (retval == 1)
+        {
+            return translate_events_from_poll(pfd.revents);
+        }
+        if (retval < 0)
+        {
+            swoole_set_last_error(errno);
+        }
+        return 0;
+    }
+
+    int revents = event_waiter(fd, events, timeout).revents;
+
+    if (revents & SW_EVENT_ERROR)
+    {
+        revents ^= SW_EVENT_ERROR;
+        if (events & SW_EVENT_READ)
+        {
+            revents |= SW_EVENT_READ;
+        }
+        if (events & SW_EVENT_WRITE)
+        {
+            revents |= SW_EVENT_WRITE;
+        }
+    }
+
+    return revents;
+}
+
+void System::init_reactor(swReactor *reactor)
+{
+    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_READ, socket_poll_read_callback);
+    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_WRITE, socket_poll_write_callback);
+    swReactor_set_handler(reactor, SW_FD_CORO_POLL | SW_EVENT_ERROR, socket_poll_error_callback);
+
+    swReactor_set_handler(reactor, SW_FD_CORO_EVENT | SW_EVENT_READ, event_waiter_read_callback);
+    swReactor_set_handler(reactor, SW_FD_CORO_EVENT | SW_EVENT_WRITE, event_waiter_write_callback);
+    swReactor_set_handler(reactor, SW_FD_CORO_EVENT | SW_EVENT_ERROR, event_waiter_error_callback);
+
+    swReactor_set_handler(reactor, SW_FD_AIO | SW_EVENT_READ, swAio_callback);
+}
+
 static void async_task_completed(swAio_event *event)
 {
     if (event->canceled)
@@ -590,3 +753,64 @@ bool coroutine::async(swAio_handler handler, swAio_event &event, double timeout)
         return true;
     }
 }
+
+struct AsyncLambdaTask
+{
+    Coroutine *co;
+    std::function<void(void)> fn;
+};
+
+static void async_lambda_handler(swAio_event *event)
+{
+    AsyncLambdaTask *task = reinterpret_cast<AsyncLambdaTask *>(event->object);
+    task->fn();
+    event->error = 0;
+    event->ret = 0;
+}
+
+static void async_lambda_callback(swAio_event *event)
+{
+    if (event->canceled)
+    {
+        return;
+    }
+    AsyncLambdaTask *task = reinterpret_cast<AsyncLambdaTask *>(event->object);
+    task->co->resume();
+}
+
+bool coroutine::async(const std::function<void(void)> &fn, double timeout)
+{
+    swTimer_node *timer = nullptr;
+    swAio_event event = {};
+
+    AsyncLambdaTask task;
+    task.co = Coroutine::get_current_safe();
+    task.fn = fn;
+
+    event.object = &task;
+    event.handler = async_lambda_handler;
+    event.callback = async_lambda_callback;
+
+    swAio_event *_ev = swAio_dispatch2(&event);
+    if (_ev == nullptr)
+    {
+        return false;
+    }
+    if (timeout > 0)
+    {
+        timer = swoole_timer_add((long) (timeout * 1000), SW_FALSE, async_task_timeout, _ev);
+    }
+    task.co->yield();
+    if (event.error == SW_ERROR_AIO_TIMEOUT)
+    {
+        return false;
+    }
+    else
+    {
+        if (timer)
+        {
+            swoole_timer_del(timer);
+        }
+        return true;
+    }
+}
