@@ -29,7 +29,7 @@ static int swPort_onRead_redis(swReactor *reactor, swListenPort *lp, swEvent *ev
 
 void swPort_init(swListenPort *port)
 {
-    port->sock = 0;
+    port->socket = nullptr;
     port->ssl = 0;
 
     //listen backlog
@@ -43,7 +43,7 @@ void swPort_init(swListenPort *port)
     port->protocol.package_length_type = 'N';
     port->protocol.package_length_size = 4;
     port->protocol.package_body_offset = 4;
-    port->protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
+    port->protocol.package_max_length = SW_INPUT_BUFFER_SIZE;
 
     port->socket_buffer_size = SwooleG.socket_buffer_size;
 
@@ -93,7 +93,7 @@ int swPort_enable_ssl_encrypt(swListenPort *ls)
 
 int swPort_listen(swListenPort *ls)
 {
-    int sock = ls->sock;
+    int sock = ls->socket->fd;
     int option = 1;
 
     //listen stream socket
@@ -196,10 +196,7 @@ void swPort_set_protocol(swServer *serv, swListenPort *ls)
     }
     else if (ls->open_mqtt_protocol)
     {
-        ls->protocol.package_length_size = SW_MQTT_MIN_LENGTH;
-        ls->protocol.package_length_offset = 0;
-        ls->protocol.package_body_offset = 0;
-        ls->protocol.get_package_length = swMqtt_get_package_length;
+        swMqtt_set_protocol(&ls->protocol);
         ls->protocol.onPackage = swReactorThread_dispatch;
         ls->onRead = swPort_onRead_check_length;
     }
@@ -214,6 +211,86 @@ void swPort_set_protocol(swServer *serv, swListenPort *ls)
     }
 }
 
+/**
+ * @description: set the swListenPort.host and swListenPort.port in swListenPort from sock
+ */
+int swPort_set_address(swListenPort *ls, int sock)
+{
+    socklen_t optlen;
+    swSocketAddress address;
+    int sock_type, sock_family;
+    char tmp[INET6_ADDRSTRLEN];
+
+    //get socket type
+    optlen = sizeof(sock_type);
+    if (getsockopt(sock, SOL_SOCKET, SO_TYPE, &sock_type, &optlen) < 0)
+    {
+        swWarn("getsockopt(%d, SOL_SOCKET, SO_TYPE) failed", sock);
+        return -1;
+    }
+    //get socket family
+#ifndef SO_DOMAIN
+    swWarn("no getsockopt(SO_DOMAIN) supports");
+    return -1;
+#else
+    optlen = sizeof(sock_family);
+    if (getsockopt(sock, SOL_SOCKET, SO_DOMAIN, &sock_family, &optlen) < 0)
+    {
+        swWarn("getsockopt(%d, SOL_SOCKET, SO_DOMAIN) failed", sock);
+        return -1;
+    }
+#endif
+
+    //get address info
+    address.len = sizeof(address.addr);
+    if (getsockname(sock, (struct sockaddr*) &address.addr, &address.len) < 0)
+    {
+        swWarn("getsockname(%d) failed", sock);
+        return -1;
+    }
+
+    swPort_init(ls);
+
+    switch (sock_family)
+    {
+    case AF_INET:
+        if (sock_type == SOCK_STREAM)
+        {
+            ls->type = SW_SOCK_TCP;
+        }
+        else
+        {
+            ls->type = SW_SOCK_UDP;
+        }
+        ls->port = ntohs(address.addr.inet_v4.sin_port);
+        strncpy(ls->host, inet_ntoa(address.addr.inet_v4.sin_addr), SW_HOST_MAXSIZE - 1);
+        break;
+    case AF_INET6:
+        if (sock_type == SOCK_STREAM)
+        {
+            ls->type = SW_SOCK_TCP6;
+        }
+        else
+        {
+            ls->type = SW_SOCK_UDP6;
+        }
+        ls->port = ntohs(address.addr.inet_v6.sin6_port);
+        inet_ntop(AF_INET6, &address.addr.inet_v6.sin6_addr, tmp, sizeof(tmp));
+        strncpy(ls->host, tmp, SW_HOST_MAXSIZE - 1);
+        break;
+    case AF_UNIX:
+        ls->type = sock_type == SOCK_STREAM ? SW_SOCK_UNIX_STREAM : SW_SOCK_UNIX_DGRAM;
+        ls->port = 0;
+        strncpy(ls->host, address.addr.un.sun_path, SW_HOST_MAXSIZE);
+        break;
+    default:
+        swWarn("Unknown socket family[%d]", sock_family);
+        break;
+    }
+
+    return 0;
+}
+
 void swPort_clear_protocol(swListenPort *ls)
 {
     ls->open_eof_check = 0;
@@ -234,10 +311,10 @@ static int swPort_onRead_raw(swReactor *reactor, swListenPort *port, swEvent *ev
     swConnection *conn = (swConnection *) _socket->object;
     char *buffer = SwooleTG.buffer_stack->str;
 
-    n = swConnection_recv(_socket, buffer, SwooleTG.buffer_stack->size, 0);
+    n = swSocket_recv(_socket, buffer, SwooleTG.buffer_stack->size, 0);
     if (n < 0)
     {
-        switch (swConnection_error(errno))
+        switch (swSocket_error(errno))
         {
         case SW_ERROR:
             swSysWarn("recv from connection#%d failed", event->fd);
@@ -344,10 +421,10 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
     swString *buffer = request->buffer;
 
     _recv_data:
-    ssize_t n = swConnection_recv(_socket, buffer->str + buffer->length, buffer->size - buffer->length, 0);
+    ssize_t n = swSocket_recv(_socket, buffer->str + buffer->length, buffer->size - buffer->length, 0);
     if (n < 0)
     {
-        switch (swConnection_error(errno))
+        switch (swSocket_error(errno))
         {
         case SW_ERROR:
             swSysWarn("recv from connection#%d failed", event->fd);
@@ -366,21 +443,21 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
         {
             _bad_request:
 #ifdef SW_HTTP_BAD_REQUEST_PACKET
-            swConnection_send(_socket, SW_STRL(SW_HTTP_BAD_REQUEST_PACKET), 0);
+            swSocket_send(_socket, SW_STRL(SW_HTTP_BAD_REQUEST_PACKET), 0);
 #endif
         }
         if (0)
         {
             _too_large:
 #ifdef SW_HTTP_REQUEST_ENTITY_TOO_LARGE_PACKET
-            swConnection_send(_socket, SW_STRL(SW_HTTP_REQUEST_ENTITY_TOO_LARGE_PACKET), 0);
+            swSocket_send(_socket, SW_STRL(SW_HTTP_REQUEST_ENTITY_TOO_LARGE_PACKET), 0);
 #endif
         }
         if (0)
         {
             _unavailable:
 #ifdef SW_HTTP_SERVICE_UNAVAILABLE_PACKET
-            swConnection_send(_socket, SW_STRL(SW_HTTP_SERVICE_UNAVAILABLE_PACKET), 0);
+            swSocket_send(_socket, SW_STRL(SW_HTTP_SERVICE_UNAVAILABLE_PACKET), 0);
 #endif
         }
         _close_fd:
@@ -474,7 +551,7 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
         if (request->content_length == 0 && (request->known_length || !request->chunked))
         {
             // send static file content directly in the reactor thread
-            if (!serv->enable_static_handler || !swHttp_static_handler(serv, request, conn))
+            if (!serv->enable_static_handler || !swServer_http_static_handler_hit(serv, request, conn))
             {
                 // dynamic request, dispatch to worker
                 swReactorThread_dispatch(protocol, _socket, buffer->str, request->header_length);
@@ -555,7 +632,7 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
             // Expect: 100-continue
             if (swHttpRequest_has_expect_header(request))
             {
-                swConnection_send(_socket, SW_STRL(SW_HTTP_100_CONTINUE_PACKET), 0);
+                swSocket_send(_socket, SW_STRL(SW_HTTP_100_CONTINUE_PACKET), 0);
             }
             else
             {
@@ -643,10 +720,16 @@ void swPort_free(swListenPort *port)
         {
             sw_free(port->ssl_option.client_cert_file);
         }
+#ifdef SW_SUPPORT_DTLS
+        if (port->dtls_sessions)
+        {
+            delete port->dtls_sessions;
+        }
+#endif
     }
 #endif
 
-    close(port->sock);
+    swSocket_free(port->socket);
 
     //remove unix socket file
     if (port->type == SW_SOCK_UNIX_STREAM || port->type == SW_SOCK_UNIX_DGRAM)
