@@ -19,8 +19,8 @@
 static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker);
 static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event);
 static int swReactorProcess_onClose(swReactor *reactor, swEvent *event);
-static int swReactorProcess_send2client(swFactory *, swSendData *);
-static int swReactorProcess_send2worker(int, const void *, int);
+static int swReactorProcess_send2client(swFactory *, swSendData *data);
+static int swReactorProcess_send2worker(swSocket *socket, const void *data, size_t length);
 static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode);
 
 #ifdef HAVE_REUSEPORT
@@ -31,7 +31,7 @@ static uint32_t heartbeat_check_lasttime = 0;
 
 static bool swServer_is_single(swServer *serv)
 {
-    return serv->worker_num == 1 && serv->task_worker_num == 0 && serv->max_request == 0 && serv->user_worker_list == NULL;
+    return serv->worker_num == 1 && serv->task_worker_num == 0 && serv->max_request == 0 && serv->user_worker_list == nullptr;
 }
 
 int swReactorProcess_create(swServer *serv)
@@ -61,27 +61,28 @@ void swReactorProcess_free(swServer *serv)
 
 int swReactorProcess_start(swServer *serv)
 {
-    swListenPort *ls;
     serv->single_thread = 1;
 
     //listen TCP
     if (serv->have_stream_sock == 1)
     {
-        LL_FOREACH(serv->listen_list, ls)
+        for (auto ls : *serv->listen_list)
         {
             if (swSocket_is_dgram(ls->type))
             {
                 continue;
             }
-            if (SwooleG.reuse_port)
+#ifdef HAVE_REUSEPORT
+            if (serv->enable_reuse_port)
             {
-                if (close(ls->sock) < 0)
+                if (close(ls->socket->fd) < 0)
                 {
-                    swSysWarn("close(%d) failed", ls->sock);
+                    swSysWarn("close(%d) failed", ls->socket->fd);
                 }
                 continue;
             }
             else
+#endif
             {
                 //listen server socket
                 if (swPort_listen(ls) < 0)
@@ -133,7 +134,7 @@ int swReactorProcess_start(swServer *serv)
     //task workers
     if (serv->task_worker_num > 0)
     {
-        if (swServer_create_task_worker(serv) < 0)
+        if (swServer_create_task_workers(serv) < 0)
         {
             return SW_ERR;
         }
@@ -155,17 +156,16 @@ int swReactorProcess_start(swServer *serv)
             swSysWarn("gmalloc[server->user_workers] failed");
             return SW_ERR;
         }
-        swUserWorker_node *user_worker;
-        LL_FOREACH(serv->user_worker_list, user_worker)
+        for (auto worker : *serv->user_worker_list)
         {
             /**
              * store the pipe object
              */
-            if (user_worker->worker->pipe_object)
+            if (worker->pipe_object)
             {
-                swServer_store_pipe_fd(serv, user_worker->worker->pipe_object);
+                swServer_store_pipe_fd(serv, worker->pipe_object);
             }
-            swManager_spawn_user_worker(serv, user_worker->worker);
+            swManager_spawn_user_worker(serv, worker);
         }
     }
 
@@ -197,7 +197,7 @@ int swReactorProcess_start(swServer *serv)
     swProcessPool_wait(&serv->gs->event_workers);
     swProcessPool_shutdown(&serv->gs->event_workers);
 
-    swManager_kill_user_worker(serv);
+    swManager_kill_user_workers(serv);
 
     if (serv->onManagerStop)
     {
@@ -213,7 +213,7 @@ static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event)
     swSendData _send;
     swServer *serv = (swServer *) reactor->ptr;
     swFactory *factory = &serv->factory;
-    swString *buffer_output;
+    swString *output_buffer;
 
     if (read(event->fd, &task, sizeof(task)) <= 0)
     {
@@ -235,16 +235,16 @@ static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event)
         break;
     case SW_SERVER_EVENT_PROXY_START:
     case SW_SERVER_EVENT_PROXY_END:
-        buffer_output = SwooleWG.buffer_output[task.info.reactor_id];
-        swString_append_ptr(buffer_output, task.data, task.info.len);
+        output_buffer = SwooleWG.output_buffer[task.info.reactor_id];
+        swString_append_ptr(output_buffer, task.data, task.info.len);
         if (task.info.type == SW_SERVER_EVENT_PROXY_END)
         {
             memcpy(&_send.info, &task.info, sizeof(_send.info));
             _send.info.type = SW_SERVER_EVENT_SEND_DATA;
-            _send.data = buffer_output->str;
-            _send.info.len = buffer_output->length;
+            _send.data = output_buffer->str;
+            _send.info.len = output_buffer->length;
             factory->finish(factory, &_send);
-            swString_clear(buffer_output);
+            swString_clear(output_buffer);
         }
         break;
     default:
@@ -255,20 +255,20 @@ static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event)
 
 static int swReactorProcess_alloc_output_buffer(int n_buffer)
 {
-    SwooleWG.buffer_output = (swString **) sw_malloc(sizeof(swString*) * n_buffer);
-    if (SwooleWG.buffer_output == NULL)
+    SwooleWG.output_buffer = (swString **) sw_malloc(sizeof(swString*) * n_buffer);
+    if (SwooleWG.output_buffer == NULL)
     {
-        swError("malloc for SwooleWG.buffer_output failed");
+        swError("malloc for SwooleWG.output_buffer failed");
         return SW_ERR;
     }
 
     int i;
     for (i = 0; i < n_buffer; i++)
     {
-        SwooleWG.buffer_output[i] = swString_new(SW_BUFFER_SIZE_BIG);
-        if (SwooleWG.buffer_output[i] == NULL)
+        SwooleWG.output_buffer[i] = swString_new(SW_BUFFER_SIZE_BIG);
+        if (SwooleWG.output_buffer[i] == NULL)
         {
-            swError("buffer_output init failed");
+            swError("output_buffer init failed");
             return SW_ERR;
         }
     }
@@ -280,9 +280,9 @@ static void swReactor_free_output_buffer(int n_buffer)
     int i;
     for (i = 0; i < n_buffer; i++)
     {
-        swString_free(SwooleWG.buffer_output[i]);
+        swString_free(SwooleWG.output_buffer[i]);
     }
-    sw_free(SwooleWG.buffer_output);
+    sw_free(SwooleWG.output_buffer);
 }
 
 static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
@@ -329,14 +329,13 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
         return SW_ERR;
     }
 
-    swListenPort *ls;
     int fdtype;
 
-    LL_FOREACH(serv->listen_list, ls)
+    for (auto ls : *serv->listen_list)
     {
         fdtype = swSocket_is_dgram(ls->type) ? SW_FD_DGRAM_SERVER : SW_FD_STREAM_SERVER;
 #ifdef HAVE_REUSEPORT
-        if (fdtype == SW_FD_STREAM_SERVER && SwooleG.reuse_port)
+        if (fdtype == SW_FD_STREAM_SERVER && serv->enable_reuse_port)
         {
             if (swReactorProcess_reuse_port(ls) < 0)
             {
@@ -346,7 +345,7 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
             }
         }
 #endif
-        if (reactor->add(reactor, ls->sock, fdtype) < 0)
+        if (reactor->add(reactor, ls->socket, SW_EVENT_READ) < 0)
         {
             return SW_ERR;
         }
@@ -380,11 +379,11 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     {
         swSocket_set_nonblock(worker->pipe_worker);
         swSocket_set_nonblock(worker->pipe_master);
-        if (reactor->add(reactor, worker->pipe_worker, SW_FD_PIPE) < 0)
+        if (reactor->add(reactor, worker->pipe_worker, SW_EVENT_READ) < 0)
         {
             return SW_ERR;
         }
-        if (reactor->add(reactor, worker->pipe_master, SW_FD_PIPE) < 0)
+        if (reactor->add(reactor, worker->pipe_master, SW_EVENT_READ) < 0)
         {
             return SW_ERR;
         }
@@ -393,19 +392,11 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     //task workers
     if (serv->task_worker_num > 0)
     {
-        swPipe *p;
-        swSocket *psock;
-        int pfd;
-
         if (serv->task_ipc_mode == SW_TASK_IPC_UNIXSOCK)
         {
             for (uint32_t i = 0; i < serv->gs->task_workers.worker_num; i++)
             {
-                p = serv->gs->task_workers.workers[i].pipe_object;
-                pfd = p->getFd(p, 1);
-                psock = swReactor_get(reactor, pfd);
-                psock->fdtype = SW_FD_PIPE;
-                swSocket_set_nonblock(pfd);
+                swSocket_set_nonblock(serv->gs->task_workers.workers[i].pipe_master);
             }
         }
     }
@@ -423,7 +414,7 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     }
 
     /**
-     * 1 second timer, update serv->gs->now
+     * 1 second timer
      */
     if ((serv->master_timer = swoole_timer_add(1000, SW_TRUE, swServer_master_onTimer, serv)) == NULL)
     {
@@ -495,11 +486,11 @@ static int swReactorProcess_onClose(swReactor *reactor, swEvent *event)
     {
         return SW_ERR;
     }
-    if (reactor->del(reactor, fd) == 0)
+    if (reactor->del(reactor, event->socket) == 0)
     {
         if (conn->close_queued)
         {
-            swReactorThread_close(reactor, fd);
+            swReactorThread_close(reactor, event->socket);
             return SW_OK; 
         }
         else 
@@ -513,28 +504,28 @@ static int swReactorProcess_onClose(swReactor *reactor, swEvent *event)
     }
 }
 
-static int swReactorProcess_send2worker(int pipe_fd, const void *data, int length)
+static int swReactorProcess_send2worker(swSocket *socket, const void *data, size_t length)
 {
     if (!SwooleTG.reactor)
     {
-        return swSocket_write_blocking(pipe_fd, data, length);
+        return swSocket_write_blocking(socket, data, length);
     }
     else
     {
-        return SwooleTG.reactor->write(SwooleTG.reactor, pipe_fd, data, length);
+        return SwooleTG.reactor->write(SwooleTG.reactor, socket, data, length);
     }
 }
 
-static int swReactorProcess_send2client(swFactory *factory, swSendData *_send)
+static int swReactorProcess_send2client(swFactory *factory, swSendData *data)
 {
     swServer *serv = (swServer *) factory->ptr;
-    int session_id = _send->info.fd;
+    int session_id = data->info.fd;
 
     swSession *session = swServer_get_session(serv, session_id);
     if (session->fd == 0)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_NOT_EXIST, "send %d byte failed, session#%d does not exist",
-                _send->info.len, session_id);
+                data->info.len, session_id);
         return SW_ERR;
     }
     //proxy
@@ -545,13 +536,13 @@ static int swReactorProcess_send2client(swFactory *factory, swSendData *_send)
         swEventData proxy_msg;
         bzero(&proxy_msg.info, sizeof(proxy_msg.info));
 
-        if (_send->info.type == SW_SERVER_EVENT_SEND_DATA)
+        if (data->info.type == SW_SERVER_EVENT_SEND_DATA)
         {
             proxy_msg.info.fd = session_id;
             proxy_msg.info.reactor_id = SwooleWG.id;
             proxy_msg.info.type = SW_SERVER_EVENT_PROXY_START;
 
-            size_t send_n = _send->info.len;
+            size_t send_n = data->info.len;
             size_t offset = 0;
 
             while (send_n > 0)
@@ -565,7 +556,7 @@ static int swReactorProcess_send2client(swFactory *factory, swSendData *_send)
                     proxy_msg.info.type = SW_SERVER_EVENT_PROXY_END;
                     proxy_msg.info.len = send_n;
                 }
-                memcpy(proxy_msg.data, _send->data + offset, proxy_msg.info.len);
+                memcpy(proxy_msg.data, data->data + offset, proxy_msg.info.len);
                 send_n -= proxy_msg.info.len;
                 offset += proxy_msg.info.len;
                 swReactorProcess_send2worker(worker->pipe_master, (const char *) &proxy_msg, sizeof(proxy_msg.info) + proxy_msg.info.len);
@@ -573,22 +564,22 @@ static int swReactorProcess_send2client(swFactory *factory, swSendData *_send)
 
             swTrace("proxy message, fd=%d, len=%ld",worker->pipe_master, sizeof(proxy_msg.info) + proxy_msg.info.len);
         }
-        else if (_send->info.type == SW_SERVER_EVENT_SEND_FILE)
+        else if (data->info.type == SW_SERVER_EVENT_SEND_FILE)
         {
-            memcpy(&proxy_msg.info, &_send->info, sizeof(proxy_msg.info));
-            memcpy(proxy_msg.data, _send->data, _send->info.len);
+            memcpy(&proxy_msg.info, &data->info, sizeof(proxy_msg.info));
+            memcpy(proxy_msg.data, data->data, data->info.len);
             return swReactorProcess_send2worker(worker->pipe_master, (const char *) &proxy_msg, sizeof(proxy_msg.info) + proxy_msg.info.len);
         }
         else
         {
-            swWarn("unkown event type[%d]", _send->info.type);
+            swWarn("unkown event type[%d]", data->info.type);
             return SW_ERR;
         }
         return SW_OK;
     }
     else
     {
-        return swFactory_finish(factory, _send);
+        return swFactory_finish(factory, data);
     }
 }
 
@@ -598,8 +589,9 @@ static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode)
     swServer *serv = (swServer *) reactor->ptr;
     swEvent notify_ev;
     swConnection *conn;
+    time_t now = time(NULL);
 
-    if (serv->gs->now < heartbeat_check_lasttime + 10)
+    if (now < heartbeat_check_lasttime + 10)
     {
         return;
     }
@@ -613,7 +605,7 @@ static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode)
     int serv_max_fd = swServer_get_maxfd(serv);
     int serv_min_fd = swServer_get_minfd(serv);
 
-    checktime = serv->gs->now - serv->heartbeat_idle_time;
+    checktime = now - serv->heartbeat_idle_time;
 
     for (fd = serv_min_fd; fd <= serv_max_fd; fd++)
     {
@@ -627,11 +619,12 @@ static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode)
 #ifdef SW_USE_OPENSSL
             if (conn->socket->ssl && conn->socket->ssl_state != SW_SSL_STATE_READY)
             {
-                swReactorThread_close(reactor, fd);
+                swReactorThread_close(reactor, conn->socket);
                 continue;
             }
 #endif
             notify_ev.fd = fd;
+            notify_ev.socket = conn->socket;
             notify_ev.reactor_id = conn->reactor_id;
             swReactorProcess_onClose(reactor, &notify_ev);
         }
@@ -641,25 +634,27 @@ static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode)
 #ifdef HAVE_REUSEPORT
 static int swReactorProcess_reuse_port(swListenPort *ls)
 {
-    //create new socket
-    int sock = swSocket_create(ls->type);
+    int sock = swSocket_create(ls->type, 1, 1);
     if (sock < 0)
     {
         swSysWarn("create socket failed");
         return SW_ERR;
     }
-    //bind address and port
-    if (swSocket_bind(sock, ls->type, ls->host, &ls->port) < 0)
+    int option = 1;
+    if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &option, sizeof(int)) != 0)
     {
         close(sock);
         return SW_ERR;
     }
-    //stream socket, set nonblock
-    if (swSocket_is_stream(ls->type))
+    ls->socket->fd = sock;
+    //bind address and port
+    if (swSocket_bind(ls->socket, ls->host, &ls->port) < 0)
     {
-        swSocket_set_nonblock(sock);
+        close(ls->socket->fd);
+        return SW_ERR;
     }
-    ls->sock = sock;
+    ls->socket->nonblock = 1;
+    ls->socket->cloexec = 1;
     return swPort_listen(ls);
 }
 #endif
