@@ -17,8 +17,12 @@
  */
 
 #include "php_swoole_cxx.h"
+#include "server.h"
 
 using namespace swoole;
+using namespace std;
+
+static unordered_map<int, swSocket *> event_socket_map;
 
 zend_class_entry *swoole_event_ce;
 static zend_object_handlers swoole_event_handlers;
@@ -145,7 +149,7 @@ static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
         php_swoole_fatal_error(E_WARNING, "%s: onRead callback handler error, fd [%d] will be removed from reactor", ZSTR_VAL(swoole_event_ce->name), swoole_convert_to_fd(&peo->zsocket));
         event->socket->object = NULL;
         swoole_event_defer(php_event_object_free, peo);
-        swoole_event_del(event->fd);
+        swoole_event_del(event->socket);
         return SW_ERR;
     }
 
@@ -161,7 +165,7 @@ static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
         php_swoole_fatal_error(E_WARNING, "%s: onWrite callback handler error, fd [%d] will be removed from reactor", ZSTR_VAL(swoole_event_ce->name), swoole_convert_to_fd(&peo->zsocket));
         event->socket->object = NULL;
         swoole_event_defer(php_event_object_free, peo);
-        swoole_event_del(event->fd);
+        swoole_event_del(event->socket);
         return SW_ERR;
     }
 
@@ -196,7 +200,7 @@ static int php_swoole_event_onError(swReactor *reactor, swEvent *event)
     }
 
     php_event_object_free(event->socket->object);
-    swoole_event_del(event->fd);
+    swoole_event_del(event->socket);
 
     return SW_OK;
 }
@@ -232,9 +236,9 @@ int php_swoole_reactor_init()
         return SW_ERR;
     }
 
-    if (SwooleG.serv)
+    if (sw_server())
     {
-        if (swIsTaskWorker() && !SwooleG.serv->task_enable_coroutine)
+        if (swIsTaskWorker() && !sw_server()->task_enable_coroutine)
         {
             php_swoole_fatal_error(E_ERROR, "Unable to use async-io in task processes, please set `task_enable_coroutine` to true");
             return SW_ERR;
@@ -472,6 +476,16 @@ static void check_reactor()
     }
 }
 
+static swSocket* get_event_socket(int socket_fd)
+{
+    auto i = event_socket_map.find(socket_fd);
+    if (i == event_socket_map.end())
+    {
+        return nullptr;
+    }
+    return i->second;
+}
+
 static PHP_FUNCTION(swoole_event_add)
 {
     zval *zfd;
@@ -479,7 +493,7 @@ static PHP_FUNCTION(swoole_event_add)
     zend_fcall_info_cache fci_cache_read = empty_fcall_info_cache;
     zend_fcall_info fci_write = empty_fcall_info;
     zend_fcall_info_cache fci_cache_write = empty_fcall_info_cache;
-    zend_long events = 0;
+    zend_long events = SW_EVENT_READ;
 
     ZEND_PARSE_PARAMETERS_START(1, 4)
         Z_PARAM_ZVAL(zfd)
@@ -491,7 +505,7 @@ static PHP_FUNCTION(swoole_event_add)
 
     if (fci_read.size == 0 && fci_write.size == 0)
     {
-        php_swoole_fatal_error(E_WARNING, "both read and write callbacks are emoty");
+        php_swoole_fatal_error(E_WARNING, "both read and write callbacks are empty");
         RETURN_FALSE;
     }
 
@@ -506,6 +520,16 @@ static PHP_FUNCTION(swoole_event_add)
         php_swoole_fatal_error(E_WARNING, "invalid socket fd [%d]", socket_fd);
         RETURN_FALSE;
     }
+    if (event_socket_map.find(socket_fd) != event_socket_map.end())
+    {
+        php_swoole_fatal_error(E_WARNING, "already exist");
+        RETURN_FALSE;
+    }
+    if (!(events & (SW_EVENT_WRITE | SW_EVENT_READ)))
+    {
+        php_swoole_fatal_error(E_WARNING, "invalid events");
+        RETURN_FALSE;
+    }
 
     php_event_object *peo = (php_event_object *) ecalloc(1, sizeof(php_event_object));
 
@@ -524,18 +548,25 @@ static PHP_FUNCTION(swoole_event_add)
     }
 
     check_reactor();
-    swSocket_set_nonblock(socket_fd); // must be nonblock
 
-    if (swoole_event_add(socket_fd, events, SW_FD_USER) < 0)
+    swSocket *socket = swSocket_new(socket_fd, SW_FD_USER);
+    if (!socket)
     {
-        php_swoole_fatal_error(E_WARNING, "swoole_event_add failed");
         RETURN_FALSE;
     }
 
-    swSocket *socket = swReactor_get(SwooleTG.reactor, socket_fd);
+    swSocket_set_nonblock(socket);
     socket->object = peo;
-    socket->nonblock = 1;
-    socket->fdtype = SW_FD_USER;
+
+    if (swoole_event_add(socket, events) < 0)
+    {
+        php_swoole_fatal_error(E_WARNING, "swoole_event_add failed");
+        efree(socket);
+        php_event_object_free(peo);
+        RETURN_FALSE;
+    }
+
+    event_socket_map[socket_fd] = socket;
 
     RETURN_LONG(socket_fd);
 }
@@ -564,8 +595,15 @@ static PHP_FUNCTION(swoole_event_write)
         RETURN_FALSE;
     }
 
+    swSocket *socket = get_event_socket(socket_fd);
+    if (socket == nullptr)
+    {
+        php_swoole_fatal_error(E_WARNING, "socket[%d] is not found in the reactor", socket_fd);
+        RETURN_FALSE;
+    }
+
     check_reactor();
-    if (swoole_event_write(socket_fd, data, len) < 0)
+    if (swoole_event_write(socket, data, len) < 0)
     {
         RETURN_FALSE;
     }
@@ -605,8 +643,8 @@ static PHP_FUNCTION(swoole_event_set)
         RETURN_FALSE;
     }
 
-    swSocket *socket = swReactor_get(SwooleTG.reactor, socket_fd);
-    if (socket->fdtype != SW_FD_USER || socket->removed)
+    swSocket *socket = get_event_socket(socket_fd);
+    if (socket == nullptr)
     {
         php_swoole_fatal_error(E_WARNING, "socket[%d] is not found in the reactor", socket_fd);
         RETURN_FALSE;
@@ -640,7 +678,7 @@ static PHP_FUNCTION(swoole_event_set)
         php_swoole_fatal_error(E_WARNING, "%s: unable to find write callback of fd [%d]", ZSTR_VAL(swoole_event_ce->name), socket_fd);
         RETURN_FALSE;
     }
-    if (swoole_event_set(socket_fd, events, SW_FD_USER) < 0)
+    if (swoole_event_set(socket, events) < 0)
     {
         php_swoole_fatal_error(E_WARNING, "%s::set failed", ZSTR_VAL(swoole_event_ce->name));
         RETURN_FALSE;
@@ -671,13 +709,17 @@ static PHP_FUNCTION(swoole_event_del)
         RETURN_FALSE;
     }
 
-    swSocket *socket = swReactor_get(SwooleTG.reactor, socket_fd);
-    if (socket->object)
+    swSocket *socket = get_event_socket(socket_fd);
+    if (!socket)
     {
-        swoole_event_defer(php_event_object_free, socket->object);
-        socket->object = NULL;
+        RETURN_FALSE;
     }
-    SW_CHECK_RETURN(swoole_event_del(socket_fd));
+    swoole_event_defer(php_event_object_free, socket->object);
+    int retval = swoole_event_del(socket);
+    event_socket_map.erase(socket_fd);
+    socket->fd = -1;
+    swSocket_free(socket);
+    RETURN_BOOL(retval == SW_OK);
 }
 
 static PHP_FUNCTION(swoole_event_defer)
@@ -832,8 +874,8 @@ static PHP_FUNCTION(swoole_event_isset)
         RETURN_FALSE;
     }
 
-    swSocket *_socket = swReactor_get(SwooleTG.reactor, socket_fd);
-    if (_socket == NULL || _socket->removed)
+    swSocket *_socket = get_event_socket(socket_fd);
+    if (_socket == nullptr || _socket->removed)
     {
         RETURN_FALSE;
     }
@@ -846,4 +888,3 @@ static PHP_FUNCTION(swoole_event_isset)
         RETURN_FALSE;
     }
 }
-
