@@ -183,6 +183,10 @@ public:
     http2_stream(http2_session *client, uint32_t _id);
     ~http2_stream();
 
+    bool send_header(size_t body_length, bool end_stream);
+    bool send_body(swString *body, bool end_stream, size_t max_frame_size, off_t offset = 0, size_t length = 0);
+    bool send_trailer();
+
     void reset(uint32_t error_code);
 };
 
@@ -241,7 +245,6 @@ static sw_inline zval* swoole_http_init_and_read_property(zend_class_entry *ce,
 }
 int swoole_http_parse_form_data(http_context *ctx, const char *boundary_str, int boundary_len);
 void swoole_http_parse_cookie(zval *array, const char *at, size_t length);
-void swoole_http_server_init_context(swServer *serv, http_context *ctx);
 
 http_context * php_swoole_http_request_get_context(zval *zobject);
 void php_swoole_http_request_set_context(zval *zobject, http_context *context);
@@ -268,19 +271,21 @@ void* php_brotli_alloc(void* opaque, size_t size);
 void php_brotli_free(void* opaque, void* address);
 #endif
 
-int swoole_websocket_onMessage(swServer *serv, swEventData *req);
-int swoole_websocket_onHandshake(swServer *serv, swListenPort *port, http_context *ctx);
-void swoole_websocket_onOpen(http_context *ctx);
-void swoole_websocket_onRequest(http_context *ctx);
-bool swoole_websocket_handshake(http_context *ctx);
-
 #ifdef SW_USE_HTTP2
-int swoole_http2_server_onFrame(swServer *serv, swConnection *conn, swEventData *req);
-int swoole_http2_server_parse(http2_session *client, const char *buf);
-int swoole_http2_server_do_response(http_context *ctx, swString *body);
-void swoole_http2_server_session_free(swConnection *conn);
+
+static sw_inline nghttp2_mem* php_nghttp2_mem()
+{
+    static nghttp2_mem mem = {
+        nullptr,
+        [](size_t size, void *mem_user_data){ return emalloc(size); },
+        [](void *ptr, void *mem_user_data){ return efree(ptr); },
+        [](size_t nmemb, size_t size, void *mem_user_data){ return ecalloc(nmemb, size); },
+        [](void *ptr, size_t size, void *mem_user_data){ return erealloc(ptr, size); }
+    };
+    return &mem;
+}
+
 void swoole_http2_response_end(http_context *ctx, zval *zdata, zval *return_value);
-int swoole_http2_server_ping(http_context *ctx);
 
 namespace swoole { namespace http2 {
 //-----------------------------------namespace begin--------------------------------------------
