@@ -14,13 +14,13 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
+#include "swoole_http_server.h"
 
 #ifdef SW_USE_HTTP2
-#include "swoole_http.h"
 
-#include "http.h"
+#include "static_handler.h"
 #include "http2.h"
+
 #include "main/php_variables.h"
 
 #include <vector>
@@ -28,9 +28,14 @@
 extern swString *swoole_http_buffer;
 
 using namespace swoole;
+using swoole::http::StaticHandler;
+using std::string;
+using swoole::coroutine::System;
 
 static std::unordered_map<int, http2_session*> http2_sessions;
 
+static bool swoole_http2_server_respond(http_context *ctx, swString *body);
+
 http2_stream::http2_stream(http2_session *client, uint32_t _id)
 {
     ctx = swoole_http_context_new(client->fd);
@@ -130,10 +135,10 @@ static ssize_t http2_build_trailer(http_context *ctx, uchar *buffer)
 
         if (!deflater)
         {
-            int ret = nghttp2_hd_deflate_new(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
+            int ret = nghttp2_hd_deflate_new2(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE, php_nghttp2_mem());
             if (ret != 0)
             {
-                swWarn("nghttp2_hd_deflate_init() failed with error: %s", nghttp2_strerror(ret));
+                swWarn("nghttp2_hd_deflate_new2() failed with error: %s", nghttp2_strerror(ret));
                 return -1;
             }
             client->deflater = deflater;
@@ -158,6 +163,58 @@ static ssize_t http2_build_trailer(http_context *ctx, uchar *buffer)
     return 0;
 }
 
+static bool swoole_http2_is_static_file(swServer *serv, http_context *ctx)
+{
+    zval *zserver = ctx->request.zserver;
+    zval *zrequest_uri = zend_hash_str_find(Z_ARR_P(zserver), ZEND_STRL("request_uri"));
+    if (zrequest_uri && Z_TYPE_P(zrequest_uri) == IS_STRING)
+    {
+        StaticHandler handler(serv, Z_STRVAL_P(zrequest_uri), Z_STRLEN_P(zrequest_uri));
+        if (!handler.hit())
+        {
+            return false;
+        }
+
+        if (handler.status_code == SW_HTTP_NOT_FOUND)
+        {
+            swString null_body;
+            null_body.length = 0;
+            null_body.str = nullptr;
+
+            ctx->response.status = SW_HTTP_NOT_FOUND;
+            swoole_http2_server_respond(ctx, &null_body);
+            return true;
+        }
+
+        auto date_str = handler.get_date();
+        auto date_str_last_modified = handler.get_date_last_modified();
+
+        zval *zheader = ctx->request.zserver;
+        swoole_http_response_set_header(ctx, ZEND_STRL("Last-Modified"), date_str.c_str(), date_str.length(), 0);
+
+        zval *zdate_if_modified_since = zend_hash_str_find(Z_ARR_P(zheader), ZEND_STRL("if-modified-since"));
+        if (zdate_if_modified_since)
+        {
+            string date_if_modified_since(Z_STRVAL_P(zdate_if_modified_since), Z_STRLEN_P(zdate_if_modified_since));
+            if (!date_if_modified_since.empty() && handler.is_modified(date_if_modified_since))
+            {
+                ctx->response.status = SW_HTTP_NOT_MODIFIED;
+                return true;
+            }
+        }
+
+        zend::string _filename(handler.get_filename_std_string());
+        zval zfilename;
+        ZVAL_STR(&zfilename, _filename.get());
+        zval retval; /* do not care the retval (the connection will be closed if failed) */
+        sw_zend_call_method_with_1_params(ctx->response.zobject, swoole_http_response_ce, NULL, "sendfile", &retval, &zfilename);
+
+        return true;
+    }
+
+    return false;
+}
+
 static void swoole_http2_onRequest(http2_session *client, http2_stream *stream)
 {
     http_context *ctx = stream->ctx;
@@ -170,14 +227,21 @@ static void swoole_http2_onRequest(http2_session *client, http2_stream *stream)
 
     ctx->request.version = SW_HTTP_OK;
 
-    add_assoc_long(zserver, "request_time", serv->gs->now);
+    if (serv->enable_static_handler && swoole_http2_is_static_file(serv, ctx))
+    {
+        zval_ptr_dtor(ctx->request.zobject);
+        zval_ptr_dtor(ctx->response.zobject);
+        return;
+    }
+
+    add_assoc_long(zserver, "request_time", time(NULL));
     add_assoc_double(zserver, "request_time_float", swoole_microtime());
     if (serv_sock)
     {
-        add_assoc_long(zserver, "server_port", swConnection_get_port(serv_sock->socket_type, &serv_sock->info));
+        add_assoc_long(zserver, "server_port", swSocket_get_port(serv_sock->socket_type, &serv_sock->info));
     }
-    add_assoc_long(zserver, "remote_port", swConnection_get_port(conn->socket_type, &conn->info));
-    add_assoc_string(zserver, "remote_addr", (char * ) swConnection_get_ip(conn->socket_type, &conn->info));
+    add_assoc_long(zserver, "remote_port", swSocket_get_port(conn->socket_type, &conn->info));
+    add_assoc_string(zserver, "remote_addr", (char * ) swSocket_get_ip(conn->socket_type, &conn->info));
     add_assoc_long(zserver, "master_time", conn->last_time);
     add_assoc_string(zserver, "server_protocol", (char * ) "HTTP/2");
 
@@ -188,6 +252,7 @@ static void swoole_http2_onRequest(http2_session *client, http2_stream *stream)
         stream->reset(SW_HTTP2_ERROR_INTERNAL_ERROR);
         php_swoole_error(E_WARNING, "%s->onRequest[v2] handler error", ZSTR_VAL(swoole_http_server_ce->name));
     }
+
     zval_ptr_dtor(&args[0]);
     zval_ptr_dtor(&args[1]);
 }
@@ -311,10 +376,10 @@ static ssize_t http2_build_header(http_context *ctx, uchar *buffer, size_t body_
     nghttp2_hd_deflater *deflater = client->deflater;
     if (!deflater)
     {
-        ret = nghttp2_hd_deflate_new(&deflater, client->header_table_size);
+        ret = nghttp2_hd_deflate_new2(&deflater, client->header_table_size, php_nghttp2_mem());
         if (ret != 0)
         {
-            swWarn("nghttp2_hd_deflate_init() failed with error: %s", nghttp2_strerror(ret));
+            swWarn("nghttp2_hd_deflate_new2() failed with error: %s", nghttp2_strerror(ret));
             return -1;
         }
         client->deflater = deflater;
@@ -346,31 +411,13 @@ int swoole_http2_server_ping(http_context *ctx)
     return ctx->send(ctx, frame, SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_FRAME_PING_PAYLOAD_SIZE) ? SW_OK : SW_ERR;
 }
 
-int swoole_http2_server_do_response(http_context *ctx, swString *body)
+bool http2_stream::send_header(size_t body_length, bool end_stream)
 {
-    http2_session *client = http2_sessions[ctx->fd];
-    http2_stream *stream = (http2_stream *) ctx->stream;
     char header_buffer[SW_BUFFER_SIZE_STD];
-    ssize_t bytes;
-
-#ifdef SW_HAVE_COMPRESSION
-    if (ctx->accept_compression)
-    {
-        if (body->length == 0 || swoole_http_response_compress(body, ctx->compression_method, ctx->compression_level) != SW_OK)
-        {
-            ctx->accept_compression = 0;
-        }
-        else
-        {
-            body = swoole_zlib_buffer;
-        }
-    }
-#endif
-
-    bytes = http2_build_header(ctx, (uchar *) header_buffer, body->length);
+    ssize_t bytes = http2_build_header(ctx, (uchar *) header_buffer, body_length);
     if (bytes < 0)
     {
-        return SW_ERR;
+        return false;
     }
 
     swString_clear(swoole_http_buffer);
@@ -389,100 +436,250 @@ int swoole_http2_server_do_response(http_context *ctx, swString *body)
      +---------------------------------------------------------------+
      */
     char frame_header[SW_HTTP2_FRAME_HEADER_SIZE];
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("trailer"), 0);
-    if (php_swoole_array_length_safe(ztrailer) == 0)
-    {
-        ztrailer = NULL;
-    }
 
-    if (!ztrailer && body->length == 0)
+    if (end_stream && body_length == 0)
     {
-        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS | SW_HTTP2_FLAG_END_STREAM, stream->id);
+        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS | SW_HTTP2_FLAG_END_STREAM, id);
     }
     else
     {
-        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS, stream->id);
+        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS, id);
     }
 
     swString_append_ptr(swoole_http_buffer, frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
     swString_append_ptr(swoole_http_buffer, header_buffer, bytes);
 
-    int flag = SW_HTTP2_FLAG_END_STREAM;
-    if (ztrailer)
-    {
-        flag = SW_HTTP2_FLAG_NONE;
-    }
-
     if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
     {
         ctx->send_header = 0;
-        return SW_ERR;
+        return false;
+    }
+
+    return true;
+}
+
+bool http2_stream::send_body(swString *body, bool end_stream, size_t max_frame_size, off_t offset, size_t length)
+{
+    char frame_header[SW_HTTP2_FRAME_HEADER_SIZE];
+    char *p = body->str + offset;
+    size_t l = length == 0 ? body->length : length;
+
+    int flag = end_stream ? SW_HTTP2_FLAG_END_STREAM : SW_HTTP2_FLAG_NONE;
+
+    while (l > 0)
+    {
+        size_t send_n;
+        int _send_flag;
+        swString_clear(swoole_http_buffer);
+        if (l > max_frame_size)
+        {
+            send_n = max_frame_size;
+            _send_flag = 0;
+        }
+        else
+        {
+            send_n = l;
+            _send_flag = flag;
+        }
+        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_DATA, send_n, _send_flag, id);
+        swString_append_ptr(swoole_http_buffer, frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
+        swString_append_ptr(swoole_http_buffer, p, send_n);
+
+        if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+        {
+            return false;
+        }
+        else
+        {
+            l -= send_n;
+            p += send_n;
+        }
+    }
+
+    return true;
+}
+
+bool http2_stream::send_trailer()
+{
+    char header_buffer[SW_BUFFER_SIZE_STD] = {};
+    char frame_header[SW_HTTP2_FRAME_HEADER_SIZE];
+
+    swString_clear(swoole_http_buffer);
+    ssize_t bytes = http2_build_trailer(ctx, (uchar *) header_buffer);
+    if (bytes > 0)
+    {
+        swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS | SW_HTTP2_FLAG_END_STREAM, id);
+        swString_append_ptr(swoole_http_buffer, frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
+        swString_append_ptr(swoole_http_buffer, header_buffer, bytes);
+        if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+        {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+static bool swoole_http2_server_respond(http_context *ctx, swString *body)
+{
+    http2_session *client = http2_sessions[ctx->fd];
+    http2_stream *stream = ctx->stream;
+
+#ifdef SW_HAVE_COMPRESSION
+    if (ctx->accept_compression)
+    {
+        if (body->length == 0 || swoole_http_response_compress(body, ctx->compression_method, ctx->compression_level) != SW_OK)
+        {
+            ctx->accept_compression = 0;
+        }
+        else
+        {
+            body = swoole_zlib_buffer;
+        }
+    }
+#endif
+
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("trailer"), 0);
+    if (php_swoole_array_length_safe(ztrailer) == 0)
+    {
+        ztrailer = nullptr;
+    }
+
+    bool end_stream = (ztrailer == nullptr);
+    if (!stream->send_header(body->length, end_stream))
+    {
+        return false;
     }
 
     /* headers has already been sent, retries are no longer allowed (even if send body failed) */
     ctx->end = 1;
 
+    bool error = false;
+
     if (body->length != 0)
     {
-        char *p = body->str;
-        size_t l = body->length;
-
-        while (l > 0)
+        if (!stream->send_body(body, end_stream, client->max_frame_size))
         {
-            size_t send_n;
-            int _send_flag;
-            swString_clear(swoole_http_buffer);
-            if (l > client->max_frame_size)
-            {
-                send_n = client->max_frame_size;
-                _send_flag = 0;
-            }
-            else
-            {
-                send_n = l;
-                _send_flag = flag;
-            }
-            swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_DATA, send_n, _send_flag, stream->id);
-            swString_append_ptr(swoole_http_buffer, frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
-            swString_append_ptr(swoole_http_buffer, p, send_n);
-
-            if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
-            {
-                ctx->close(ctx);
-                return SW_ERR;
-            }
-            else
-            {
-                l -= send_n;
-                p += send_n;
-            }
+            error = true;
+        }
+        else
+        {
+            client->send_window -= body->length; // TODO: flow control?
         }
-
-        client->send_window -= body->length;    // TODO: flow control?
     }
 
-    if (ztrailer)
+    if (!error && ztrailer)
     {
-        swString_clear(swoole_http_buffer);
-        memset(header_buffer, 0, sizeof(header_buffer));
-        bytes = http2_build_trailer(ctx, (uchar *) header_buffer);
-        if (bytes > 0)
+        if (!stream->send_trailer())
         {
-            swHttp2_set_frame_header(frame_header, SW_HTTP2_TYPE_HEADERS, bytes, SW_HTTP2_FLAG_END_HEADERS | SW_HTTP2_FLAG_END_STREAM, stream->id);
-            swString_append_ptr(swoole_http_buffer, frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
-            swString_append_ptr(swoole_http_buffer, header_buffer, bytes);
-            if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
-            {
-                ctx->close(ctx);
-                return SW_ERR;
-            }
+            error = true;
         }
     }
 
-    client->streams.erase(stream->id);
-    delete stream;
+    if (error)
+    {
+        ctx->close(ctx);
+    }
+    else
+    {
+        client->streams.erase(stream->id);
+        delete stream;
+    }
 
-    return SW_OK;
+    return !error;
+}
+
+static bool http2_context_sendfile(http_context* ctx, const char *file, uint32_t l_file, off_t offset, size_t length)
+{
+    http2_session *client = http2_sessions[ctx->fd];
+    http2_stream *stream = (http2_stream *) ctx->stream;
+    swString *body;
+
+#ifdef SW_HAVE_COMPRESSION
+    ctx->accept_compression = 0;
+#endif
+    if (swoole_coroutine_is_in())
+    {
+        body = System::read_file(file, false);
+        if (!body)
+        {
+            return false;
+        }
+        if (!ctx->stream)
+        {
+            /* closed */
+            return false;
+        }
+    }
+    else
+    {
+        int fd = open(file, O_RDONLY);
+        if (fd < 0)
+        {
+            return false;
+        }
+        body = swoole_sync_readfile_eof(fd);
+        close(fd);
+        if (!body)
+        {
+            return false;
+        }
+    }
+    body->length = SW_MIN(length, body->length);
+
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("trailer"), 0);
+    if (php_swoole_array_length_safe(ztrailer) == 0)
+    {
+        ztrailer = nullptr;
+    }
+
+    const char* mimetype = swoole::mime_type::get(file).c_str();
+    swoole_http_response_set_header(ctx, ZEND_STRL("content-type"), mimetype, strlen(mimetype), 0);
+
+    bool end_stream = (ztrailer == nullptr);
+    if (!stream->send_header(length, end_stream))
+    {
+        return false;
+    }
+
+    /* headers has already been sent, retries are no longer allowed (even if send body failed) */
+    ctx->end = 1;
+
+    bool error = false;
+
+    if (body->length > 0)
+    {
+        if (!stream->send_body(body, end_stream, client->max_frame_size, offset, length))
+        {
+            error = true;
+        }
+        else
+        {
+            client->send_window -= length; // TODO: flow control?
+        }
+    }
+
+    swString_free(body);
+
+    if (!error && ztrailer)
+    {
+        if (!stream->send_trailer())
+        {
+            error = true;
+        }
+    }
+
+    if (error)
+    {
+        ctx->close(ctx);
+    }
+    else
+    {
+        client->streams.erase(stream->id);
+        delete stream;
+    }
+
+    return true;
 }
 
 static int http2_parse_header(http2_session *client, http_context *ctx, int flags, const char *in, size_t inlen)
@@ -491,10 +688,10 @@ static int http2_parse_header(http2_session *client, http_context *ctx, int flag
 
     if (!inflater)
     {
-        int ret = nghttp2_hd_inflate_new(&inflater);
+        int ret = nghttp2_hd_inflate_new2(&inflater, php_nghttp2_mem());
         if (ret != 0)
         {
-            swWarn("nghttp2_hd_inflate_init() failed, Error: %s[%d]", nghttp2_strerror(ret), ret);
+            swWarn("nghttp2_hd_inflate_new2() failed, Error: %s[%d]", nghttp2_strerror(ret), ret);
             return SW_ERR;
         }
         client->inflater = inflater;
@@ -885,10 +1082,11 @@ int swoole_http2_server_onFrame(swServer *serv, swConnection *conn, swEventData
         client->default_ctx->http2 = true;
         client->default_ctx->stream = (http2_stream *) -1;
         client->default_ctx->keepalive = true;
+        client->default_ctx->sendfile = http2_context_sendfile;
     }
 
     zval zdata;
-    php_swoole_get_recv_data(serv, &zdata, req, NULL, 0);
+    php_swoole_get_recv_data(serv, &zdata, req);
     swoole_http2_server_parse(client, Z_STRVAL(zdata));
     zval_ptr_dtor(&zdata);
 
@@ -919,7 +1117,7 @@ void swoole_http2_response_end(http_context *ctx, zval *zdata, zval *return_valu
         http_body.str = NULL;
     }
 
-    RETURN_BOOL(swoole_http2_server_do_response(ctx, &http_body) == SW_OK);
+    RETURN_BOOL(swoole_http2_server_respond(ctx, &http_body));
 }
 
 #endif
