@@ -21,7 +21,7 @@
 #include "coroutine_c_api.h"
 #include "swoole_http_client.h"
 
-#include "mime_types.h"
+#include "mime_type.h"
 #include "base64.h"
 
 #ifdef SW_HAVE_BROTLI
@@ -150,8 +150,8 @@ public:
         }
 
         array_init(return_value);
-        add_assoc_string(return_value, "host", (char *) swConnection_get_ip(socket->get_type(), &sa));
-        add_assoc_long(return_value, "port", swConnection_get_port(socket->get_type(), &sa));
+        add_assoc_string(return_value, "address", (char *) swSocket_get_ip(socket->get_type(), &sa));
+        add_assoc_long(return_value, "port", swSocket_get_port(socket->get_type(), &sa));
     }
 
     void getpeername(zval *return_value)
@@ -164,10 +164,26 @@ public:
         }
 
         array_init(return_value);
-        add_assoc_string(return_value, "host", (char *) swConnection_get_ip(socket->get_type(), &sa));
-        add_assoc_long(return_value, "port", swConnection_get_port(socket->get_type(), &sa));
+        add_assoc_string(return_value, "address", (char *) swSocket_get_ip(socket->get_type(), &sa));
+        add_assoc_long(return_value, "port", swSocket_get_port(socket->get_type(), &sa));
     }
 
+#ifdef SW_USE_OPENSSL
+    void getpeercert(zval *return_value)
+    {
+        auto cert = socket->ssl_get_peer_cert();
+        if (cert.empty())
+        {
+            ZVAL_FALSE(return_value);
+            return;
+        }
+        else
+        {
+            ZVAL_STRINGL(return_value, cert.c_str(), cert.length());
+        }
+    }
+#endif
+
     ~http_client();
 
 private:
@@ -289,6 +305,8 @@ static PHP_METHOD(swoole_http_client_coro, setData);
 static PHP_METHOD(swoole_http_client_coro, addFile);
 static PHP_METHOD(swoole_http_client_coro, addData);
 static PHP_METHOD(swoole_http_client_coro, execute);
+static PHP_METHOD(swoole_http_client_coro, getsockname);
+static PHP_METHOD(swoole_http_client_coro, getpeername);
 static PHP_METHOD(swoole_http_client_coro, get);
 static PHP_METHOD(swoole_http_client_coro, post);
 static PHP_METHOD(swoole_http_client_coro, download);
@@ -297,6 +315,9 @@ static PHP_METHOD(swoole_http_client_coro, getHeaders);
 static PHP_METHOD(swoole_http_client_coro, getCookies);
 static PHP_METHOD(swoole_http_client_coro, getStatusCode);
 static PHP_METHOD(swoole_http_client_coro, getHeaderOut);
+#ifdef SW_USE_OPENSSL
+static PHP_METHOD(swoole_http_client_coro, getPeerCert);
+#endif
 static PHP_METHOD(swoole_http_client_coro, upgrade);
 static PHP_METHOD(swoole_http_client_coro, push);
 static PHP_METHOD(swoole_http_client_coro, recv);
@@ -317,6 +338,8 @@ static const zend_function_entry swoole_http_client_coro_methods[] =
     PHP_ME(swoole_http_client_coro, addFile, arginfo_swoole_http_client_coro_addFile, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, addData, arginfo_swoole_http_client_coro_addData, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, execute, arginfo_swoole_http_client_coro_execute, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http_client_coro, getpeername, arginfo_swoole_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http_client_coro, getsockname, arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, get, arginfo_swoole_http_client_coro_get, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, post, arginfo_swoole_http_client_coro_post, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, download, arginfo_swoole_http_client_coro_download, ZEND_ACC_PUBLIC)
@@ -325,6 +348,9 @@ static const zend_function_entry swoole_http_client_coro_methods[] =
     PHP_ME(swoole_http_client_coro, getCookies, arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, getStatusCode, arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, getHeaderOut, arginfo_swoole_void, ZEND_ACC_PUBLIC)
+#ifdef SW_USE_OPENSSL
+    PHP_ME(swoole_http_client_coro, getPeerCert, arginfo_swoole_void, ZEND_ACC_PUBLIC)
+#endif
     PHP_ME(swoole_http_client_coro, upgrade, arginfo_swoole_http_client_coro_upgrade, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, push, arginfo_swoole_http_client_coro_push, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_client_coro, recv, arginfo_swoole_http_client_coro_recv, ZEND_ACC_PUBLIC)
@@ -866,7 +892,7 @@ bool http_client::send()
     socket->check_bound_co(SW_EVENT_WRITE);
 
     //clear errno
-    SwooleG.error = 0;
+    swoole_set_last_error(0);
     //alloc buffer
     swString *buffer = socket->get_write_buffer();
     swString_clear(buffer);
@@ -880,6 +906,14 @@ bool http_client::send()
     zcookies = sw_zend_read_property(swoole_http_client_coro_ce, zobject, ZEND_STRL("cookies"), 0);
     z_download_file = sw_zend_read_property_not_null(swoole_http_client_coro_ce, zobject, ZEND_STRL("downloadFile"), 0);
 
+    // ============   host   ============
+    zend::string str_host;
+
+    if ((ZVAL_IS_ARRAY(zheaders)) && ((zvalue = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("Host"))) || (zvalue = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("host")))))
+    {
+        str_host = zvalue;
+    }
+
     // ============ download ============
     if (z_download_file)
     {
@@ -943,18 +977,13 @@ bool http_client::send()
     if (socket->http_proxy)
 #endif
     {
-        zend::string str_host;
         const static char *pre = "http://";
         char *_host = (char *) host.c_str();
         size_t _host_len = host.length();
-        if (ZVAL_IS_ARRAY(zheaders))
+        if (str_host.get())
         {
-            if ((zvalue = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("Host"))))
-            {
-                str_host = zvalue;
-                _host = str_host.val();
-                _host_len = str_host.len();
-            }
+            _host = str_host.val();
+            _host_len = str_host.len();
         }
         size_t proxy_uri_len = path.length() + _host_len + strlen(pre) + 10;
         char *proxy_uri = (char*) emalloc(proxy_uri_len);
@@ -977,20 +1006,25 @@ bool http_client::send()
 
     // As much as possible to ensure that Host is the first header.
     // See: http://tools.ietf.org/html/rfc7230#section-5.4
-    if ((ZVAL_IS_ARRAY(zheaders)) && ((zvalue = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("Host"))) || (zvalue = zend_hash_str_find(Z_ARRVAL_P(zheaders), ZEND_STRL("host")))))
+    if (str_host.get())
     {
-        zend::string str_value(zvalue);
-        http_client_swString_append_headers(buffer, ZEND_STRL("Host"), str_value.val(), str_value.len());
+        http_client_swString_append_headers(buffer, ZEND_STRL("Host"), str_host.val(), str_host.len());
     }
     else
     {
         // See: https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.23
-        const std::string *_host = &host;
+        const std::string *_host;
         std::string __host;
-        if (port != 80 && port != 443)
+#ifndef SW_USE_OPENSSL
+        if (port != 80)
+#else
+        if (!ssl ? port != 80 : port != 443)
+#endif
         {
             __host = cpp_string::format("%s:%u", host.c_str(), port);
             _host = &__host;
+        } else {
+            _host = &host;
         }
         http_client_swString_append_headers(buffer, ZEND_STRL("Host"), _host->c_str(), _host->length());
     }
@@ -1393,7 +1427,8 @@ bool http_client::recv(double timeout)
     }
     if (!socket || !socket->is_connect())
     {
-        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION);
+        swoole_set_last_error(SW_ERROR_CLIENT_NO_CONNECTION);
+        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), swoole_get_last_error());
         zend_update_property_string(swoole_http_client_coro_ce, zobject, ZEND_STRL("errMsg"), "connection is not available");
         zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         return false;
@@ -1439,7 +1474,8 @@ void http_client::recv(zval *zframe, double timeout)
     ZVAL_FALSE(zframe);
     if (!socket || !socket->is_connect())
     {
-        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION);
+        swoole_set_last_error(SW_ERROR_CLIENT_NO_CONNECTION);
+        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), swoole_get_last_error());
         zend_update_property_string(swoole_http_client_coro_ce, zobject, ZEND_STRL("errMsg"), "connection is not available");
         zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         return;
@@ -1466,6 +1502,7 @@ void http_client::recv(zval *zframe, double timeout)
 #else
         php_swoole_websocket_frame_unpack(&msg, zframe);
 #endif
+        zend_update_property_long(swoole_websocket_frame_ce, zframe, ZEND_STRL("fd"), socket->get_fd());
     }
 }
 
@@ -1552,15 +1589,17 @@ bool http_client::push(zval *zdata, zend_long opcode, uint8_t flags)
 {
     if (!websocket)
     {
+        swoole_set_last_error(SW_ERROR_WEBSOCKET_HANDSHAKE_FAILED);
         php_swoole_fatal_error(E_WARNING, "websocket handshake failed, cannot push data");
-        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), SwooleG.error = SW_ERROR_WEBSOCKET_HANDSHAKE_FAILED);
+        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), swoole_get_last_error());
         zend_update_property_string(swoole_http_client_coro_ce, zobject, ZEND_STRL("errMsg"), "websocket handshake failed, cannot push data");
         zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_CONNECT_FAILED);
         return false;
     }
     if (!socket || !socket->is_connect())
     {
-        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION);
+        swoole_set_last_error(SW_ERROR_CLIENT_NO_CONNECTION);
+        zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("errCode"), swoole_get_last_error());
         zend_update_property_string(swoole_http_client_coro_ce, zobject, ZEND_STRL("errMsg"), "connection is not available");
         zend_update_property_long(swoole_http_client_coro_ce, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         return false;
@@ -1693,7 +1732,7 @@ static void php_swoole_http_client_coro_free_object(zend_object *object)
 
 static zend_object *php_swoole_http_client_coro_create_object(zend_class_entry *ce)
 {
-    http_client_coro *hcc = (http_client_coro *) ecalloc(1, sizeof(http_client_coro) + zend_object_properties_size(ce));
+    http_client_coro *hcc = (http_client_coro *) zend_object_alloc(sizeof(http_client_coro), ce);
     zend_object_std_init(&hcc->std, ce);
     object_properties_init(&hcc->std, ce);
     hcc->std.handlers = &swoole_http_client_coro_handlers;
@@ -1960,7 +1999,7 @@ static PHP_METHOD(swoole_http_client_coro, addFile)
     }
     if (l_type == 0)
     {
-        type = (char *) swoole_mime_type_get(path);
+        type = (char *) swoole::mime_type::get(path).c_str();
         l_type = strlen(type);
     }
     if (l_filename == 0)
@@ -2189,3 +2228,23 @@ static PHP_METHOD(swoole_http_client_coro, getHeaderOut)
     http_client *phc = php_swoole_get_phc(ZEND_THIS);
     phc->get_header_out(return_value);
 }
+
+static PHP_METHOD(swoole_http_client_coro, getsockname)
+{
+    http_client *phc = php_swoole_get_phc(ZEND_THIS);
+    phc->getsockname(return_value);
+}
+
+static PHP_METHOD(swoole_http_client_coro, getpeername)
+{
+    http_client *phc = php_swoole_get_phc(ZEND_THIS);
+    phc->getpeername(return_value);
+}
+
+#ifdef SW_USE_OPENSSL
+static PHP_METHOD(swoole_http_client_coro, getPeerCert)
+{
+    http_client *phc = php_swoole_get_phc(ZEND_THIS);
+    phc->getpeercert(return_value);
+}
+#endif
