@@ -14,8 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
-#include "swoole_http.h"
+#include "swoole_http_server.h"
 
 extern "C"
 {
@@ -32,7 +31,6 @@ extern "C"
 #include "main/php_variables.h"
 
 #include "websocket.h"
-#include "connection.h"
 #include "base64.h"
 
 #ifdef SW_HAVE_ZLIB
@@ -271,7 +269,7 @@ static void php_swoole_http_request_free_object(zend_object *object)
 
 static zend_object *php_swoole_http_request_create_object(zend_class_entry *ce)
 {
-    http_request_t *request = (http_request_t *) ecalloc(1, sizeof(http_request_t) + zend_object_properties_size(ce));
+    http_request_t *request = (http_request_t *) zend_object_alloc(sizeof(http_request_t), ce);
     zend_object_std_init(&request->std, ce);
     object_properties_init(&request->std, ce);
     request->std.handlers = &swoole_http_request_handlers;
@@ -288,6 +286,7 @@ ZEND_END_ARG_INFO()
 const zend_function_entry swoole_http_request_methods[] =
 {
     PHP_ME(swoole_http_request, rawContent, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
+    PHP_MALIAS(swoole_http_request, getContent, rawContent, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_request, getData, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_request, __destruct, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_FE_END
