@@ -14,9 +14,9 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
-#include "swoole_http.h"
-#include "mime_types.h"
+#include "swoole_http_server.h"
+
+#include "mime_type.h"
 
 extern "C"
 {
@@ -31,7 +31,6 @@ extern "C"
 }
 
 #include "websocket.h"
-#include "connection.h"
 #include "base64.h"
 
 #ifdef SW_HAVE_ZLIB
@@ -52,7 +51,7 @@ using swoole::coroutine::Socket;
 zend_class_entry *swoole_http_response_ce;
 static zend_object_handlers swoole_http_response_handlers;
 
-static void http_build_header(http_context *, swString *response, int body_length);
+static void http_build_header(http_context *, swString *response, size_t body_length);
 
 static inline void http_header_key_format(char *key, int length)
 {
@@ -165,7 +164,7 @@ static void php_swoole_http_response_free_object(zend_object *object)
 
 static zend_object *php_swoole_http_response_create_object(zend_class_entry *ce)
 {
-    http_response_t *response = (http_response_t *) ecalloc(1, sizeof(http_response_t) + zend_object_properties_size(ce));
+    http_response_t *response = (http_response_t *) zend_object_alloc(sizeof(http_response_t), ce);
     zend_object_std_init(&response->std, ce);
     object_properties_init(&response->std, ce);
     response->std.handlers = &swoole_http_response_handlers;
@@ -248,6 +247,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_redirect, 0, 0, 1)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_create, 0, 0, 1)
+    ZEND_ARG_INFO(0, server)
     ZEND_ARG_INFO(0, fd)
 ZEND_END_ARG_INFO()
 
@@ -337,7 +337,7 @@ static PHP_METHOD(swoole_http_response, write)
     {
         ctx->send_chunked = 1;
         swString_clear(http_buffer);
-        http_build_header(ctx, http_buffer, -1);
+        http_build_header(ctx, http_buffer, 0);
         if (!ctx->send(ctx, http_buffer->str, http_buffer->length))
         {
             ctx->send_chunked = 0;
@@ -378,7 +378,7 @@ static PHP_METHOD(swoole_http_response, write)
     RETURN_BOOL(ctx->send(ctx, http_buffer->str, http_buffer->length));
 }
 
-static void http_build_header(http_context *ctx, swString *response, int body_length)
+static void http_build_header(http_context *ctx, swString *response, size_t body_length)
 {
     char *buf = SwooleTG.buffer_stack->str;
     size_t l_buf = SwooleTG.buffer_stack->size;
@@ -454,17 +454,37 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
         (void)type;
     }
 
+    //http cookies
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("cookie"), 0);
+    if (ZVAL_IS_ARRAY(zcookie))
+    {
+        zval *zvalue;
+        SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(zcookie), zvalue)
+        {
+            if (Z_TYPE_P(zvalue) != IS_STRING)
+            {
+                continue;
+            }
+            swString_append_ptr(response, ZEND_STRL("Set-Cookie: "));
+            swString_append_ptr(response, Z_STRVAL_P(zvalue), Z_STRLEN_P(zvalue));
+            swString_append_ptr(response, ZEND_STRL("\r\n"));
+        }
+        SW_HASHTABLE_FOREACH_END();
+    }
+
     if (!(header_flag & HTTP_HEADER_SERVER))
     {
         swString_append_ptr(response, ZEND_STRL("Server: " SW_HTTP_SERVER_SOFTWARE "\r\n"));
     }
-    //websocket protocol
+
+    // websocket protocol (subsequent header info is unnecessary)
     if (ctx->upgrade == 1)
     {
         swString_append_ptr(response, ZEND_STRL("\r\n"));
         ctx->send_header = 1;
         return;
     }
+
     if (!(header_flag & HTTP_HEADER_CONNECTION))
     {
         if (ctx->keepalive)
@@ -490,6 +510,7 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
 
     if (ctx->send_chunked)
     {
+        SW_ASSERT(body_length == 0);
         if (!(header_flag & HTTP_HEADER_TRANSFER_ENCODING))
         {
             swString_append_ptr(response, ZEND_STRL("Transfer-Encoding: chunked\r\n"));
@@ -504,27 +525,9 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
             body_length = swoole_zlib_buffer->length;
         }
 #endif
-        n = sw_snprintf(buf, l_buf, "Content-Length: %d\r\n", body_length);
+        n = sw_snprintf(buf, l_buf, "Content-Length: %zu\r\n", body_length);
         swString_append_ptr(response, buf, n);
     }
-
-    //http cookies
-    zval *zcookie = sw_zend_read_property(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("cookie"), 0);
-    if (ZVAL_IS_ARRAY(zcookie))
-    {
-        zval *zvalue;
-        SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(zcookie), zvalue)
-        {
-            if (Z_TYPE_P(zvalue) != IS_STRING)
-            {
-                continue;
-            }
-            swString_append_ptr(response, ZEND_STRL("Set-Cookie: "));
-            swString_append_ptr(response, Z_STRVAL_P(zvalue), Z_STRLEN_P(zvalue));
-            swString_append_ptr(response, ZEND_STRL("\r\n"));
-        }
-        SW_HASHTABLE_FOREACH_END();
-    }
 #ifdef SW_HAVE_COMPRESSION
     //http compress
     if (ctx->accept_compression)
@@ -906,16 +909,9 @@ static PHP_METHOD(swoole_http_response, sendfile)
 
     if (ctx->send_chunked)
     {
-        php_swoole_fatal_error(E_ERROR, "can't use sendfile when Http-Chunk is enabled");
+        php_swoole_fatal_error(E_WARNING, "can't use sendfile when HTTP chunk is enabled");
         RETURN_FALSE;
     }
-#ifdef SW_USE_HTTP2
-    if (ctx->http2)
-    {
-        php_swoole_fatal_error(E_WARNING, "can't use sendfile when http2 connection is established");
-        RETURN_FALSE;
-    }
-#endif
 
     char *file;
     size_t l_file;
@@ -939,19 +935,14 @@ static PHP_METHOD(swoole_http_response, sendfile)
         php_swoole_sys_error(E_WARNING, "stat(%s) failed", file);
         RETURN_FALSE;
     }
-    if (file_stat.st_size == 0)
-    {
-        php_swoole_sys_error(E_WARNING, "can't send empty file[%s]", file);
-        RETURN_FALSE;
-    }
-    if (file_stat.st_size <= offset)
+    if (file_stat.st_size < offset)
     {
         php_swoole_error(E_WARNING, "parameter $offset[" ZEND_LONG_FMT "] exceeds the file size", offset);
         RETURN_FALSE;
     }
     if (length > file_stat.st_size - offset)
     {
-        php_swoole_sys_error(E_WARNING, "parameter $length[" ZEND_LONG_FMT "] exceeds the file size", length);
+        php_swoole_error(E_WARNING, "parameter $length[" ZEND_LONG_FMT "] exceeds the file size", length);
         RETURN_FALSE;
     }
     if (length == 0)
@@ -959,12 +950,14 @@ static PHP_METHOD(swoole_http_response, sendfile)
         length = file_stat.st_size - offset;
     }
 
-#ifdef SW_HAVE_COMPRESSION
-    ctx->accept_compression = 0;
+#ifdef SW_USE_HTTP2
+    if (!ctx->http2)
 #endif
-
     if (!ctx->send_header)
     {
+#ifdef SW_HAVE_COMPRESSION
+        ctx->accept_compression = 0;
+#endif
         swString *http_buffer = http_get_write_buffer(ctx);
 
         swString_clear(http_buffer);
@@ -972,7 +965,7 @@ static PHP_METHOD(swoole_http_response, sendfile)
         zval *zheader = sw_zend_read_and_convert_property_array(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("header"), 0);
         if (!zend_hash_str_exists(Z_ARRVAL_P(zheader), ZEND_STRL("Content-Type")))
         {
-            add_assoc_string(zheader, "Content-Type", (char *) swoole_mime_type_get(file));
+            add_assoc_string(zheader, "Content-Type", (char *) swoole::mime_type::get(file).c_str());
         }
 
         http_build_header(ctx, http_buffer, length);
@@ -984,10 +977,13 @@ static PHP_METHOD(swoole_http_response, sendfile)
         }
     }
 
-    if (!ctx->sendfile(ctx, file, l_file, offset, length))
+    if (length != 0)
     {
-        ctx->close(ctx);
-        RETURN_FALSE;
+        if (!ctx->sendfile(ctx, file, l_file, offset, length))
+        {
+            ctx->close(ctx);
+            RETURN_FALSE;
+        }
     }
 
     ctx->end = 1;
@@ -1232,7 +1228,7 @@ static PHP_METHOD(swoole_http_response, push)
     http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_SESSION_CLOSED;
+        swoole_set_last_error(SW_ERROR_SESSION_CLOSED);
         RETURN_FALSE;
     }
     if (UNEXPECTED(!ctx->co_socket || !ctx->upgrade))
@@ -1282,7 +1278,7 @@ static PHP_METHOD(swoole_http_response, close)
     http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_SESSION_CLOSED;
+        swoole_set_last_error(SW_ERROR_SESSION_CLOSED);
         RETURN_FALSE;
     }
     RETURN_BOOL(ctx->close(ctx));
@@ -1293,7 +1289,7 @@ static PHP_METHOD(swoole_http_response, recv)
     http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_SESSION_CLOSED;
+        swoole_set_last_error(SW_ERROR_SESSION_CLOSED);
         RETURN_FALSE;
     }
     if (UNEXPECTED(!ctx->co_socket || !ctx->upgrade))
@@ -1315,7 +1311,7 @@ static PHP_METHOD(swoole_http_response, recv)
 
     if (retval < 0)
     {
-        SwooleG.error = sock->errCode;
+        swoole_set_last_error(sock->errCode);
         RETURN_FALSE;
     }
     else if (retval == 0)
@@ -1348,24 +1344,49 @@ static PHP_METHOD(swoole_http_response, detach)
 
 static PHP_METHOD(swoole_http_response, create)
 {
-    if (!SwooleG.serv || !SwooleG.serv->gs->start)
+    zval *zserver = NULL;
+    zend_long fd;
+    swServer *serv;
+
+    if (ZEND_NUM_ARGS() == 1)
+    {
+        ZEND_PARSE_PARAMETERS_START(1, 1)
+            Z_PARAM_LONG(fd)
+        ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+        serv = sw_server();
+    }
+    else
+    {
+        ZEND_PARSE_PARAMETERS_START(2, 2)
+            Z_PARAM_OBJECT_OF_CLASS(zserver, swoole_server_ce)
+            Z_PARAM_LONG(fd)
+        ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+        serv = php_swoole_server_get_and_check_server(zserver);
+    }
+
+    if (serv == nullptr || !serv->gs->start)
     {
         php_swoole_fatal_error(E_WARNING, "server is not running");
         RETURN_FALSE;
     }
 
-    zend_long fd;
-
-    ZEND_PARSE_PARAMETERS_START(1, 1)
-        Z_PARAM_LONG(fd)
-    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
-
     http_context *ctx = (http_context *) ecalloc(1, sizeof(http_context));
+    if (!ctx)
+    {
+        RETURN_FALSE;
+    }
 
     ctx->fd = (int) fd;
     ctx->keepalive = 1;
 
-    swoole_http_server_init_context(SwooleG.serv, ctx);
+    swoole_http_server_init_context(sw_server(), ctx);
+
+    if (sw_unlikely(swoole_http_buffer == nullptr))
+    {
+        php_swoole_http_server_init_global_variant();
+    }
 
     object_init_ex(return_value, swoole_http_response_ce);
     php_swoole_http_response_set_context(return_value, ctx);
