@@ -14,8 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
-#include "swoole_http.h"
+#include "swoole_http_server.h"
 #include "websocket.h"
 
 #include "main/rfc1867.h"
@@ -75,7 +74,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
     swoole_http_server_init_context(serv, ctx);
 
     zval *zdata = &ctx->request.zdata;
-    php_swoole_get_recv_data(serv, zdata, req, NULL, 0);
+    php_swoole_get_recv_data(serv, zdata, req);
 
     swTraceLog(SW_TRACE_SERVER, "http request from %d with %d bytes: <<EOF\n%.*s\nEOF", fd, (int) Z_STRLEN_P(zdata), (int) Z_STRLEN_P(zdata), Z_STRVAL_P(zdata));
 
@@ -103,10 +102,10 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
         swConnection *serv_sock = swServer_connection_get(serv, conn->server_fd);
         if (serv_sock)
         {
-            add_assoc_long(zserver, "server_port", swConnection_get_port(serv_sock->socket_type, &serv_sock->info));
+            add_assoc_long(zserver, "server_port", swSocket_get_port(serv_sock->socket_type, &serv_sock->info));
         }
-        add_assoc_long(zserver, "remote_port", swConnection_get_port(conn->socket_type, &conn->info));
-        add_assoc_string(zserver, "remote_addr", (char *) swConnection_get_ip(conn->socket_type, &conn->info));
+        add_assoc_long(zserver, "remote_port", swSocket_get_port(conn->socket_type, &conn->info));
+        add_assoc_string(zserver, "remote_addr", (char *) swSocket_get_ip(conn->socket_type, &conn->info));
         add_assoc_long(zserver, "master_time", conn->last_time);
     } while (0);
 
@@ -331,7 +330,7 @@ bool http_context_send_data(http_context* ctx, const char *data, size_t length)
 {
     swServer *serv = (swServer *) ctx->private_data;
     ssize_t ret = serv->send(serv, ctx->fd, (void*) data, length);
-    if (ret < 0 && SwooleG.error == SW_ERROR_OUTPUT_SEND_YIELD)
+    if (ret < 0 && swoole_get_last_error() == SW_ERROR_OUTPUT_SEND_YIELD)
     {
         zval yield_data, return_value;
         ZVAL_STRINGL(&yield_data, data, length);
