@@ -14,8 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
-#include "swoole_http.h"
+#include "swoole_http_server.h"
 
 #include "http.h"
 #ifdef SW_USE_HTTP2
@@ -66,58 +65,74 @@ class http_server
 {
 public:
     Socket *socket;
-    map<string, php_swoole_fci *> handlers;
-    php_swoole_fci *default_handler;
+    zend_fcall_info_cache *default_handler;
+    map<string, zend_fcall_info_cache> handlers;
+    zval zcallbacks;
     bool running;
-    std::list<Socket *> receivers;
+    std::list<Socket *> clients;
 
     /* options */
+    bool http_parse_cookie :1;
+    bool http_parse_post :1;
+    bool http_parse_files :1;
+#ifdef SW_HAVE_COMPRESSION
+    bool http_compression :1;
+#endif
 #ifdef SW_HAVE_ZLIB
-    bool websocket_compression;
+    bool websocket_compression :1;
+#endif
+    char *upload_tmp_dir;
+#ifdef SW_HAVE_COMPRESSION
+    uint8_t http_compression_level;
 #endif
 
     http_server(enum swSocket_type type)
     {
         socket = new Socket(type);
         default_handler = nullptr;
+        array_init(&zcallbacks);
         running = true;
 
+        http_parse_cookie = true;
+        http_parse_post = true;
+        http_parse_files = false;
+#ifdef SW_HAVE_COMPRESSION
+        http_compression = true;
+        http_compression_level = SW_Z_BEST_SPEED;
+#endif
 #ifdef SW_HAVE_ZLIB
         websocket_compression = false;
 #endif
+        upload_tmp_dir = sw_strdup("/tmp");
     }
 
-    void set_handler(string pattern, php_swoole_fci *fci)
+    ~http_server()
     {
+        sw_free(upload_tmp_dir);
+    }
+
+    void set_handler(string pattern, zval *zcallback, const zend_fcall_info_cache *fci_cache)
+    {
+        handlers[pattern] = *fci_cache;
         if (pattern == "/")
         {
-            if (default_handler)
-            {
-                sw_zend_fci_cache_discard(&default_handler->fci_cache);
-                efree(default_handler);
-            }
-            default_handler = fci;
+            default_handler = &handlers[pattern];
         }
-        else
-        {
-            auto find_fci = handlers.find(pattern);
-            if (find_fci != handlers.end())
-            {
-                sw_zend_fci_cache_discard(&find_fci->second->fci_cache);
-                efree(find_fci->second);
-            }
-            handlers[pattern] = fci;
-        }
-        sw_zend_fci_cache_persist(&fci->fci_cache);
+        Z_ADDREF_P(zcallback);
+        add_assoc_zval_ex(&zcallbacks, pattern.c_str(), pattern.length(), zcallback);
     }
 
-    php_swoole_fci* get_handler(http_context *ctx)
+    zend_fcall_info_cache* get_handler(http_context *ctx)
     {
         for (auto i = handlers.begin(); i != handlers.end(); i++)
         {
+            if (&i->second == default_handler)
+            {
+                continue;
+            }
             if (swoole_strcasect(ctx->request.path, ctx->request.path_len, i->first.c_str(), i->first.length()))
             {
-                return i->second;
+                return &i->second;
             }
         }
         return default_handler;
@@ -126,11 +141,12 @@ public:
     http_context* create_context(Socket *conn, zval *zconn)
     {
         http_context *ctx = swoole_http_context_new(conn->get_fd());
-        ctx->parse_body = 1;
-        ctx->parse_cookie = 1;
+        ctx->parse_body = http_parse_post;
+        ctx->parse_cookie = http_parse_cookie;
+        ctx->parse_files = http_parse_files;
 #ifdef SW_HAVE_COMPRESSION
-        ctx->enable_compression = 1;
-        ctx->compression_level = SW_Z_BEST_SPEED;
+        ctx->enable_compression = http_compression;
+        ctx->compression_level = http_compression_level;
 #endif
 #ifdef SW_HAVE_ZLIB
         ctx->websocket_compression = websocket_compression;
@@ -140,7 +156,7 @@ public:
         ctx->send = http_context_send_data;
         ctx->sendfile = http_context_sendfile;
         ctx->close = http_context_disconnect;
-        ctx->upload_tmp_dir = "/tmp";
+        ctx->upload_tmp_dir = upload_tmp_dir;
 
         swoole_http_parser *parser = &ctx->parser;
         parser->data = ctx;
@@ -217,7 +233,7 @@ static const zend_function_entry swoole_http_server_coro_methods[] =
 
 static zend_object *php_swoole_http_server_coro_create_object(zend_class_entry *ce)
 {
-    http_server_coro_t *hsc = (http_server_coro_t *) ecalloc(1, sizeof(http_server_coro_t) + zend_object_properties_size(ce));
+    http_server_coro_t *hsc = (http_server_coro_t *) zend_object_alloc(sizeof(http_server_coro_t), ce);
     zend_object_std_init(&hsc->std, ce);
     object_properties_init(&hsc->std, ce);
     hsc->std.handlers = &swoole_http_server_coro_handlers;
@@ -264,16 +280,7 @@ static void php_swoole_http_server_coro_free_object(zend_object *object)
     if (hsc->server)
     {
         http_server *hs = hsc->server;
-        if (hs->default_handler)
-        {
-            sw_zend_fci_cache_discard(&hs->default_handler->fci_cache);
-            efree(hs->default_handler);
-        }
-        for (auto i = hs->handlers.begin(); i != hs->handlers.end(); i++)
-        {
-            sw_zend_fci_cache_discard(&i->second->fci_cache);
-            efree(i->second);
-        }
+        zval_ptr_dtor(&hs->zcallbacks);
         delete hs;
     }
     zend_object_std_dtor(&hsc->std);
@@ -288,6 +295,13 @@ void php_swoole_http_server_coro_minit(int module_number)
     SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_http_server_coro);
     SW_SET_CLASS_CUSTOM_OBJECT(swoole_http_server_coro, php_swoole_http_server_coro_create_object, php_swoole_http_server_coro_free_object, http_server_coro_t, std);
     swoole_http_server_coro_ce->ce_flags |= ZEND_ACC_FINAL;
+    swoole_http_server_coro_handlers.get_gc = [](sw_zend7_object *object, zval **gc_data, int *gc_count)
+    {
+        http_server_coro_t *hs = php_swoole_http_server_coro_fetch_object(SW_Z7_OBJ_P(object));
+        *gc_data = &hs->server->zcallbacks;
+        *gc_count = 1;
+        return zend_std_get_properties(object);
+    };
 
     zend_declare_property_long(swoole_http_server_coro_ce, ZEND_STRL("fd"), -1, ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_http_server_coro_ce, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
@@ -341,32 +355,27 @@ static PHP_METHOD(swoole_http_server_coro, __construct)
         zend_throw_exception_ex(swoole_exception_ce, sock->errCode, "bind(%s:%d) failed", host, (int) port);
         RETURN_FALSE;
     }
-
-#ifdef SW_USE_OPENSSL
-    /**
-     * Do not initialize ssl in listen method
-     */
-    sock->open_ssl = false;
+    //check ssl
+    if (ssl)
+    {
+#ifndef SW_USE_OPENSSL
+        zend_throw_exception_ex(
+            swoole_exception_ce,
+            EPROTONOSUPPORT, "you must configure with `--enable-openssl` to support ssl connection when compiling Swoole"
+        );
+        RETURN_FALSE;
+#else
+        /* we have to call ssl_check_context after user setProtocols */
+        zval *zsettings = sw_zend_read_and_convert_property_array(swoole_http_server_coro_ce, ZEND_THIS, ZEND_STRL("settings"), 0);
+        add_assoc_bool(zsettings, "open_ssl", 1);
 #endif
+    }
     if (!sock->listen())
     {
         http_server_set_error(ZEND_THIS, sock);
         zend_throw_exception_ex(swoole_exception_ce, sock->errCode, "listen() failed");
         RETURN_FALSE;
     }
-    //check ssl
-#ifndef SW_USE_OPENSSL
-    if (ssl)
-    {
-        zend_throw_exception_ex(
-            swoole_exception_ce,
-            EPROTONOSUPPORT, "you must configure with `--enable-openssl` to support ssl connection when compiling Swoole"
-        );
-        RETURN_FALSE;
-    }
-#else
-    sock->open_ssl = ssl;
-#endif
 
     zend_update_property_long(swoole_http_server_coro_ce, ZEND_THIS, ZEND_STRL("fd"), sock->get_fd());
     zend_update_property_long(swoole_http_server_coro_ce, ZEND_THIS, ZEND_STRL("port"), sock->get_bind_port());
@@ -378,15 +387,16 @@ static PHP_METHOD(swoole_http_server_coro, handle)
     size_t pattern_len;
 
     http_server *hs = http_server_get_object(Z_OBJ_P(ZEND_THIS));
-    php_swoole_fci *fci = (php_swoole_fci *) ecalloc(1, sizeof(php_swoole_fci));
+    zend_fcall_info fci;
+    zend_fcall_info_cache fci_cache;
 
     ZEND_PARSE_PARAMETERS_START(2, 2)
         Z_PARAM_STRING(pattern, pattern_len)
-        Z_PARAM_FUNC(fci->fci, fci->fci_cache)
+        Z_PARAM_FUNC(fci, fci_cache)
     ZEND_PARSE_PARAMETERS_END();
 
     string key(pattern, pattern_len);
-    hs->set_handler(key, fci);
+    hs->set_handler(key, ZEND_CALL_ARG(execute_data, 2), &fci_cache);
 }
 
 static PHP_METHOD(swoole_http_server_coro, set)
@@ -412,33 +422,81 @@ static PHP_METHOD(swoole_http_server_coro, set)
 static PHP_METHOD(swoole_http_server_coro, start)
 {
     http_server *hs = http_server_get_object(Z_OBJ_P(ZEND_THIS));
+    Socket *sock = hs->socket;
 
-    auto sock = hs->socket;
+    /* get callback fci cache */
     char *func_name = NULL;
     zend_fcall_info_cache fci_cache;
     zval zcallback;
     ZVAL_STRING(&zcallback, "onAccept");
-
     if (!sw_zend_is_callable_ex(&zcallback, ZEND_THIS, 0, &func_name, NULL, &fci_cache, NULL))
     {
-        php_swoole_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
+        php_swoole_fatal_error(E_CORE_ERROR, "function '%s' is not callable", func_name);
         return;
     }
     efree(func_name);
 
-    zval zsocket;
-
+    /* check settings */
     zval *zsettings = sw_zend_read_and_convert_property_array(swoole_http_server_coro_ce, ZEND_THIS, ZEND_STRL("settings"), 0);
     php_swoole_socket_set_protocol(hs->socket, zsettings);
-
-#ifdef SW_HAVE_ZLIB
     HashTable *vht = Z_ARRVAL_P(zsettings);
     zval *ztmp;
+    //parse cookie header
+    if (php_swoole_array_get_value(vht, "http_parse_cookie", ztmp))
+    {
+        hs->http_parse_cookie = zval_is_true(ztmp);
+    }
+    //parse x-www-form-urlencoded form data
+    if (php_swoole_array_get_value(vht, "http_parse_post", ztmp))
+    {
+        hs->http_parse_post = zval_is_true(ztmp);
+    }
+    //parse multipart/form-data file uploads
+    if (php_swoole_array_get_value(vht, "http_parse_files", ztmp))
+    {
+        hs->http_parse_files = zval_is_true(ztmp);
+    }
+#ifdef SW_HAVE_COMPRESSION
+    //http content compression
+    if (php_swoole_array_get_value(vht, "http_compression", ztmp))
+    {
+        hs->http_compression = zval_is_true(ztmp);
+    }
+    if (php_swoole_array_get_value(vht, "http_compression_level", ztmp) || php_swoole_array_get_value(vht, "http_gzip_level", ztmp))
+    {
+        zend_long level = zval_get_long(ztmp);
+        if (level > UINT8_MAX)
+        {
+            level = UINT8_MAX;
+        }
+        else if (level < 0)
+        {
+            level = 0;
+        }
+        hs->http_compression_level = level;
+    }
+#endif
+#ifdef SW_HAVE_ZLIB
     if (php_swoole_array_get_value(vht, "websocket_compression", ztmp))
     {
         hs->websocket_compression = zval_is_true(ztmp);
     }
 #endif
+    //temporary directory for HTTP uploaded file.
+    if (php_swoole_array_get_value(vht, "upload_tmp_dir", ztmp))
+    {
+        zend::string str_v(ztmp);
+        if (php_swoole_create_dir(str_v.val(), str_v.len()) < 0)
+        {
+            php_swoole_fatal_error(E_ERROR, "Unable to create upload_tmp_dir[%s]", str_v.val());
+            return;
+        }
+        if (hs->upload_tmp_dir)
+        {
+            sw_free(hs->upload_tmp_dir);
+        }
+        hs->upload_tmp_dir = str_v.dup();
+    }
 
     php_swoole_http_server_init_global_variant();
 
@@ -447,6 +505,7 @@ static PHP_METHOD(swoole_http_server_coro, start)
         auto conn = sock->accept();
         if (conn)
         {
+            zval zsocket;
             php_swoole_init_socket_object(&zsocket, conn);
             long cid = PHPCoroutine::create(&fci_cache, 1, &zsocket);
             zval_dtor(&zsocket);
@@ -506,15 +565,25 @@ static PHP_METHOD(swoole_http_server_coro, onAccept)
     size_t total_bytes = 0;
     http_context *ctx = nullptr;
 
+    hs->clients.push_front(sock);
+    auto client_iterator = hs->clients.begin();
+
+#ifdef SW_USE_OPENSSL
+    if (sock->open_ssl)
+    {
+        if (!sock->ssl_handshake())
+        {
+            goto _handshake_failed;
+        }
+    }
+#endif
+
     while (true)
     {
         ssize_t retval;
         if (ctx != nullptr || total_bytes == 0)
         {
-            hs->receivers.push_front(sock);
-            auto receiver = hs->receivers.begin();
             retval = sock->recv(buffer->str + total_bytes, buffer->size - total_bytes);
-            hs->receivers.erase(receiver);
 
             if (sw_unlikely(retval <= 0))
             {
@@ -569,13 +638,18 @@ static PHP_METHOD(swoole_http_server_coro, onAccept)
         }
 
 #ifdef SW_USE_HTTP2
-        if (ctx->parser.method == PHP_HTTP_NOT_IMPLEMENTED
-                && memcmp(buffer->str, SW_HTTP2_PRI_STRING, sizeof(SW_HTTP2_PRI_STRING) - 1) == 0)
+        if (
+            ctx->parser.method == PHP_HTTP_NOT_IMPLEMENTED &&
+            total_bytes >= (sizeof(SW_HTTP2_PRI_STRING) - 1) &&
+            memcmp(buffer->str, SW_HTTP2_PRI_STRING, sizeof(SW_HTTP2_PRI_STRING) - 1) == 0
+        )
         {
             buffer->length = total_bytes - (sizeof(SW_HTTP2_PRI_STRING) - 1);
             buffer->offset = buffer->length == 0 ? 0 : (sizeof(SW_HTTP2_PRI_STRING) - 1);
             hs->recv_http2_frame(ctx);
-            return;
+            /* ownership of ctx has been transferred */
+            ctx = nullptr;
+            break;
         }
 #endif
 
@@ -602,20 +676,20 @@ static PHP_METHOD(swoole_http_server_coro, onAccept)
         add_assoc_long(zserver, "remote_port", (zend_long) sock->get_port());
         add_assoc_string(zserver, "remote_addr", (char *) sock->get_ip());
 
-        php_swoole_fci *fci = hs->get_handler(ctx);
-        zval args[2] = { *ctx->request.zobject, *ctx->response.zobject };
+        zend_fcall_info_cache *fci_cache = hs->get_handler(ctx);
+        zval args[2] = {*ctx->request.zobject, *ctx->response.zobject};
         bool keep_alive = swoole_http_should_keep_alive(&ctx->parser) && !ctx->websocket;
 
-        if (fci)
+        if (fci_cache)
         {
-            if (UNEXPECTED(!zend::function::call(&fci->fci_cache, 2, args, NULL, 0)))
+            if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, NULL, 0)))
             {
                 php_swoole_error(E_WARNING, "handler error");
             }
         }
         else
         {
-            ctx->response.status = 404;
+            ctx->response.status = SW_HTTP_NOT_FOUND;
         }
 
         zval_dtor(&args[0]);
@@ -633,6 +707,15 @@ static PHP_METHOD(swoole_http_server_coro, onAccept)
         zval_dtor(ctx->request.zobject);
         zval_dtor(ctx->response.zobject);
     }
+
+#ifdef SW_USE_OPENSSL
+    _handshake_failed:
+#endif
+    /* notice: do not erase the element when server is shutting down */
+    if (hs->running)
+    {
+        hs->clients.erase(client_iterator);
+    }
 }
 
 static PHP_METHOD(swoole_http_server_coro, shutdown)
@@ -640,10 +723,12 @@ static PHP_METHOD(swoole_http_server_coro, shutdown)
     http_server *hs = http_server_get_object(Z_OBJ_P(ZEND_THIS));
     hs->running = false;
     hs->socket->cancel(SW_EVENT_READ);
-    while (!hs->receivers.empty())
+    /* accept has been canceled, we only need to traverse once */
+    for (auto client : hs->clients)
     {
-        hs->receivers.back()->close();
+        client->close();
     }
+    hs->clients.clear();
 }
 
 #ifdef SW_USE_HTTP2
@@ -661,12 +746,12 @@ static void http2_server_onRequest(http2_session *session, http2_stream *stream)
     add_assoc_string(zserver, "remote_addr", (char * ) sock->get_ip());
     add_assoc_string(zserver, "server_protocol", (char * ) "HTTP/2");
 
-    php_swoole_fci *fci = hs->get_handler(ctx);
-    zval args[2] = { *ctx->request.zobject, *ctx->response.zobject };
+    zend_fcall_info_cache *fci_cache = hs->get_handler(ctx);
+    zval args[2] = {*ctx->request.zobject, *ctx->response.zobject};
 
-    if (fci)
+    if (fci_cache)
     {
-        if (UNEXPECTED(!zend::function::call(&fci->fci_cache, 2, args, NULL, SwooleG.enable_coroutine)))
+        if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, NULL, SwooleG.enable_coroutine)))
         {
             stream->reset(SW_HTTP2_ERROR_INTERNAL_ERROR);
             php_swoole_error(E_WARNING, "%s->onRequest[v2] handler error", ZSTR_VAL(swoole_http_server_ce->name));
