@@ -52,9 +52,9 @@ using swoole::coroutine::Socket;
 
 extern "C"
 {
-static PHP_METHOD(swoole_runtime, enableStrictMode);
 static PHP_METHOD(swoole_runtime, enableCoroutine);
 static PHP_METHOD(swoole_runtime, getHookFlags);
+static PHP_METHOD(swoole_runtime, setHookFlags);
 static PHP_FUNCTION(swoole_sleep);
 static PHP_FUNCTION(swoole_usleep);
 static PHP_FUNCTION(swoole_time_nanosleep);
@@ -85,6 +85,10 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_runtime_enableCoroutine, 0, 0, 0)
     ZEND_ARG_INFO(0, flags)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_runtime_setHookFlags, 0, 0, 1)
+    ZEND_ARG_INFO(0, flags)
+ZEND_END_ARG_INFO()
+
 static zend_class_entry *swoole_runtime_ce;
 
 static php_stream_ops socket_ops
@@ -148,9 +152,9 @@ extern "C"
 
 static const zend_function_entry swoole_runtime_methods[] =
 {
-    PHP_ME(swoole_runtime, enableStrictMode, arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_runtime, enableCoroutine, arginfo_swoole_runtime_enableCoroutine, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_runtime, getHookFlags, arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_runtime, setHookFlags, arginfo_swoole_runtime_setHookFlags, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
@@ -177,26 +181,6 @@ void php_swoole_runtime_minit(int module_number)
     swoole_proc_open_init(module_number);
 }
 
-static auto block_io_functions = {
-    "sleep",
-    "usleep",
-    "time_nanosleep",
-    "time_sleep_until",
-    "file_get_contents",
-    "curl_init",
-    "stream_select",
-    "pcntl_fork",
-    "popen",
-    "socket_select",
-    "gethostbyname",
-};
-
-static auto block_io_classes = {
-    "redis", "pdo", "mysqli",
-};
-
-static bool enable_strict_mode = false;
-
 struct real_func
 {
     zend_function *function;
@@ -235,20 +219,6 @@ void php_swoole_runtime_rshutdown()
     function_table = nullptr;
 }
 
-static PHP_METHOD(swoole_runtime, enableStrictMode)
-{
-    php_swoole_fatal_error(E_DEPRECATED, "Swoole\\Runtime::enableStrictMode is deprecated, it will be removed in v4.5.0");
-    for (auto f : block_io_functions)
-    {
-        zend_disable_function((char *) f, strlen((char *) f));
-    }
-    for (auto c : block_io_classes)
-    {
-        zend_disable_class((char *) c, strlen((char *) c));
-    }
-    enable_strict_mode = true;
-}
-
 static inline char *parse_ip_address_ex(const char *str, size_t str_len, int *portno, int get_err, zend_string **err)
 {
     char *colon;
@@ -348,7 +318,7 @@ static ssize_t socket_read(php_stream *stream, char *buf, size_t count)
     /**
      * sock->errCode != ETIMEDOUT : Compatible with sync blocking IO
      */
-    stream->eof = (nr_bytes == 0 || (nr_bytes == -1 && sock->errCode != ETIMEDOUT && swConnection_error(sock->errCode) == SW_CLOSE));
+    stream->eof = (nr_bytes == 0 || (nr_bytes == -1 && sock->errCode != ETIMEDOUT && swSocket_error(sock->errCode) == SW_CLOSE));
     if (nr_bytes > 0)
     {
         php_stream_notify_progress_increment(PHP_STREAM_CONTEXT(stream), nr_bytes, 0);
@@ -467,9 +437,10 @@ static int socket_stat(php_stream *stream, php_stream_statbuf *ssb)
 
 static inline int socket_connect(php_stream *stream, Socket *sock, php_stream_xport_param *xparam)
 {
-    char *host = NULL;
-    int portno = 0;
+    char *host = NULL, *bindto = NULL;
+    int portno = 0, bindport = 0;
     int ret = 0;
+    zval *tmpzval = NULL;
     char *ip_address = NULL;
 
     if (UNEXPECTED(sock->get_fd() < 0))
@@ -496,6 +467,31 @@ static inline int socket_connect(php_stream *stream, Socket *sock, php_stream_xp
     {
         return FAILURE;
     }
+    if (PHP_STREAM_CONTEXT(stream) && (tmpzval = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "socket", "bindto")) != NULL)
+    {
+        if (Z_TYPE_P(tmpzval) != IS_STRING)
+        {
+            if (xparam->want_errortext)
+            {
+                xparam->outputs.error_text = strpprintf(0, "local_addr context option is not a string.");
+            }
+            efree(ip_address);
+            return FAILURE;
+        }
+        bindto = parse_ip_address_ex(Z_STRVAL_P(tmpzval), Z_STRLEN_P(tmpzval), &bindport, xparam->want_errortext, &xparam->outputs.error_text);
+        if (bindto == NULL)
+        {
+            efree(ip_address);
+            return FAILURE;
+        }
+        if (!sock->bind(bindto, bindport))
+        {
+            efree(ip_address);
+            efree(bindto);
+            return FAILURE;
+        }
+    }
+    
     if (xparam->inputs.timeout)
     {
         sock->set_timeout(xparam->inputs.timeout, SW_TIMEOUT_CONNECT);
@@ -513,6 +509,10 @@ static inline int socket_connect(php_stream *stream, Socket *sock, php_stream_xp
     {
         efree(ip_address);
     }
+    if (bindto)
+    {
+        efree(bindto);
+    }
     return ret;
 }
 
@@ -572,6 +572,18 @@ static inline int socket_accept(php_stream *stream, Socket *sock, php_stream_xpo
 
     Socket *clisock = sock->accept();
 
+#ifdef SW_USE_OPENSSL
+    if (clisock != nullptr && clisock->open_ssl)
+    {
+        if (!clisock->ssl_handshake())
+        {
+            sock->errCode = clisock->errCode;
+            delete clisock;
+            clisock = nullptr;
+        }
+    }
+#endif
+
     if (clisock == nullptr)
     {
         error = sock->errCode;
@@ -974,6 +986,8 @@ static php_stream *socket_create(
         return NULL;
     }
 
+    sock->set_zero_copy(true);
+
     abstract = (php_swoole_netstream_data_t*) ecalloc(1, sizeof(*abstract));
     abstract->socket = sock;
     abstract->stream.socket = sock->get_fd();
@@ -1041,11 +1055,6 @@ static php_stream *socket_create(
 
 bool PHPCoroutine::enable_hook(int flags)
 {
-    if (sw_unlikely(enable_strict_mode))
-    {
-        php_swoole_fatal_error(E_ERROR, "unable to enable the coroutine mode after you enable the strict mode");
-        return false;
-    }
     if (!hook_init)
     {
         HashTable *xport_hash = php_stream_xport_get_hash();
@@ -1169,9 +1178,12 @@ bool PHPCoroutine::enable_hook(int flags)
     {
         if (hook_flags & SW_HOOK_TLS)
         {
-            if (ori_factory.tls != nullptr) {
+            if (ori_factory.tls != nullptr)
+            {
                 php_stream_xport_register("tls", ori_factory.tls);
-            } else {
+            }
+            else
+            {
                 php_stream_xport_unregister("tls");
             }
         }
@@ -1264,8 +1276,8 @@ bool PHPCoroutine::enable_hook(int flags)
         if (hook_flags & SW_HOOK_BLOCKING_FUNCTION)
         {
             SW_UNHOOK_FUNC(gethostbyname);
-            unhook_func(ZEND_STRL("exec"));
-            unhook_func(ZEND_STRL("shell_exec"));
+            SW_UNHOOK_FUNC(exec);
+            SW_UNHOOK_FUNC(shell_exec);
         }
     }
 
@@ -1289,16 +1301,16 @@ bool PHPCoroutine::enable_hook(int flags)
     {
         if (hook_flags & SW_HOOK_CURL)
         {
-            unhook_func(ZEND_STRL("curl_init"));
-            unhook_func(ZEND_STRL("curl_setopt"));
-            unhook_func(ZEND_STRL("curl_setopt_array"));
-            unhook_func(ZEND_STRL("curl_exec"));
-            unhook_func(ZEND_STRL("curl_getinfo"));
-            unhook_func(ZEND_STRL("curl_errno"));
-            unhook_func(ZEND_STRL("curl_error"));
-            unhook_func(ZEND_STRL("curl_reset"));
-            unhook_func(ZEND_STRL("curl_close"));
-            unhook_func(ZEND_STRL("curl_multi_getcontent"));
+            SW_UNHOOK_FUNC(curl_init);
+            SW_UNHOOK_FUNC(curl_setopt);
+            SW_UNHOOK_FUNC(curl_setopt_array);
+            SW_UNHOOK_FUNC(curl_exec);
+            SW_UNHOOK_FUNC(curl_getinfo);
+            SW_UNHOOK_FUNC(curl_errno);
+            SW_UNHOOK_FUNC(curl_error);
+            SW_UNHOOK_FUNC(curl_reset);
+            SW_UNHOOK_FUNC(curl_close);
+            SW_UNHOOK_FUNC(curl_multi_getcontent);
         }
     }
 
@@ -1314,7 +1326,7 @@ bool PHPCoroutine::disable_hook()
 static PHP_METHOD(swoole_runtime, enableCoroutine)
 {
     zval *zflags = nullptr;
-    /*TODO: enable SW_HOOK_CURL by default after curl handler completed */
+    /*TODO:[v4.6] enable SW_HOOK_CURL by default after curl handler completed */
     zend_long flags = SW_HOOK_ALL;
 
     ZEND_PARSE_PARAMETERS_START(0, 2)
@@ -1351,6 +1363,17 @@ static PHP_METHOD(swoole_runtime, getHookFlags)
     RETURN_LONG(hook_flags);
 }
 
+static PHP_METHOD(swoole_runtime, setHookFlags)
+{
+    zend_long flags = SW_HOOK_ALL;
+
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_LONG(flags)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    RETURN_BOOL(PHPCoroutine::enable_hook(flags));
+}
+
 static PHP_FUNCTION(swoole_sleep)
 {
     zend_long num;
@@ -1525,7 +1548,8 @@ static void stream_array_to_fd_set(zval *stream_array, std::unordered_map<int, s
         {
             i->second.events |= event;
         }
-    } ZEND_HASH_FOREACH_END();
+    }
+    ZEND_HASH_FOREACH_END();
 }
 
 static int stream_array_emulate_read_fd_set(zval *stream_array)
