@@ -14,7 +14,7 @@
   +----------------------------------------------------------------------+
  */
 
-#include "php_swoole_cxx.h"
+#include "swoole_server.h"
 
 using namespace std;
 using namespace swoole;
@@ -120,7 +120,7 @@ static void php_swoole_server_port_free_object(zend_object *object)
 
 static zend_object *php_swoole_server_port_create_object(zend_class_entry *ce)
 {
-    server_port_t *server_port = (server_port_t *) ecalloc(1, sizeof(server_port_t) + zend_object_properties_size(ce));
+    server_port_t *server_port = (server_port_t *) zend_object_alloc(sizeof(server_port_t), ce);
     zend_object_std_init(&server_port->std, ce);
     object_properties_init(&server_port->std, ce);
     server_port->std.handlers = &swoole_server_port_handlers;
@@ -197,7 +197,7 @@ void php_swoole_server_port_minit(int module_number)
 /**
  * [Master-Process]
  */
-static ssize_t php_swoole_server_length_func(swProtocol *protocol, swSocket *conn, char *data, uint32_t length)
+static ssize_t php_swoole_server_length_func(swProtocol *protocol, swSocket *conn, const char *data, uint32_t length)
 {
     swServer *serv = (swServer *) protocol->private_data_2;
     swServer_lock(serv);
@@ -781,7 +781,7 @@ static PHP_METHOD(swoole_server_port, getCallback)
 static PHP_METHOD(swoole_server_port, getSocket)
 {
     swListenPort *port = php_swoole_server_port_get_and_check_ptr(ZEND_THIS);
-    php_socket *socket_object = swoole_convert_to_socket(port->sock);
+    php_socket *socket_object = swoole_convert_to_socket(port->socket->fd);
     if (!socket_object)
     {
         RETURN_FALSE;
