@@ -42,6 +42,8 @@ static PHP_METHOD(swoole_socket_coro, bind);
 static PHP_METHOD(swoole_socket_coro, listen);
 static PHP_METHOD(swoole_socket_coro, accept);
 static PHP_METHOD(swoole_socket_coro, connect);
+static PHP_METHOD(swoole_socket_coro, checkLiveness);
+static PHP_METHOD(swoole_socket_coro, peek);
 static PHP_METHOD(swoole_socket_coro, recv);
 static PHP_METHOD(swoole_socket_coro, send);
 static PHP_METHOD(swoole_socket_coro, sendFile);
@@ -53,6 +55,9 @@ static PHP_METHOD(swoole_socket_coro, sendto);
 static PHP_METHOD(swoole_socket_coro, getOption);
 static PHP_METHOD(swoole_socket_coro, setOption);
 static PHP_METHOD(swoole_socket_coro, setProtocol);
+#ifdef SW_USE_OPENSSL
+static PHP_METHOD(swoole_socket_coro, sslHandshake);
+#endif
 static PHP_METHOD(swoole_socket_coro, shutdown);
 static PHP_METHOD(swoole_socket_coro, close);
 static PHP_METHOD(swoole_socket_coro, cancel);
@@ -84,6 +89,13 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_connect, 0, 0, 1)
     ZEND_ARG_INFO(0, timeout)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_checkLiveness, 0, 0, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_peek, 0, 0, 0)
+    ZEND_ARG_INFO(0, length)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_recv, 0, 0, 0)
     ZEND_ARG_INFO(0, length)
     ZEND_ARG_INFO(0, timeout)
@@ -144,27 +156,32 @@ ZEND_END_ARG_INFO()
 
 static const zend_function_entry swoole_socket_coro_methods[] =
 {
-    PHP_ME(swoole_socket_coro, __construct, arginfo_swoole_socket_coro_construct, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, bind,        arginfo_swoole_socket_coro_bind,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, listen,      arginfo_swoole_socket_coro_listen,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, accept,      arginfo_swoole_socket_coro_accept,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, connect,     arginfo_swoole_socket_coro_connect,   ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, recv,        arginfo_swoole_socket_coro_recv,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, recvPacket,  arginfo_swoole_socket_coro_recvPacket,  ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, send,        arginfo_swoole_socket_coro_send,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, sendFile,    arginfo_swoole_socket_coro_sendFile,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, recvAll,     arginfo_swoole_socket_coro_recv,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, sendAll,     arginfo_swoole_socket_coro_send,      ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, recvfrom,    arginfo_swoole_socket_coro_recvfrom,  ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, sendto,      arginfo_swoole_socket_coro_sendto,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, getOption,   arginfo_swoole_socket_coro_getOption, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, setProtocol, arginfo_swoole_socket_coro_setProtocol, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, setOption,   arginfo_swoole_socket_coro_setOption, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, shutdown,    arginfo_swoole_socket_coro_shutdown,  ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, cancel,      arginfo_swoole_socket_coro_cancel,    ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, close,       arginfo_swoole_void,                  ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, getpeername, arginfo_swoole_void,                  ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_socket_coro, getsockname, arginfo_swoole_void,                  ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, __construct,   arginfo_swoole_socket_coro_construct,     ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, bind,          arginfo_swoole_socket_coro_bind,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, listen,        arginfo_swoole_socket_coro_listen,        ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, accept,        arginfo_swoole_socket_coro_accept,        ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, connect,       arginfo_swoole_socket_coro_connect,       ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, checkLiveness, arginfo_swoole_socket_coro_checkLiveness, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, peek,          arginfo_swoole_socket_coro_peek,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, recv,          arginfo_swoole_socket_coro_recv,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, recvPacket,    arginfo_swoole_socket_coro_recvPacket,    ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, send,          arginfo_swoole_socket_coro_send,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, sendFile,      arginfo_swoole_socket_coro_sendFile,      ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, recvAll,       arginfo_swoole_socket_coro_recv,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, sendAll,       arginfo_swoole_socket_coro_send,          ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, recvfrom,      arginfo_swoole_socket_coro_recvfrom,      ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, sendto,        arginfo_swoole_socket_coro_sendto,        ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, getOption,     arginfo_swoole_socket_coro_getOption,     ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, setProtocol,   arginfo_swoole_socket_coro_setProtocol,   ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, setOption,     arginfo_swoole_socket_coro_setOption,     ZEND_ACC_PUBLIC)
+#ifdef SW_USE_OPENSSL
+    PHP_ME(swoole_socket_coro, sslHandshake,  arginfo_swoole_void,                      ZEND_ACC_PUBLIC)
+#endif
+    PHP_ME(swoole_socket_coro, shutdown,      arginfo_swoole_socket_coro_shutdown,      ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, cancel,        arginfo_swoole_socket_coro_cancel,        ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, close,         arginfo_swoole_void,                      ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, getpeername,   arginfo_swoole_void,                      ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, getsockname,   arginfo_swoole_void,                      ZEND_ACC_PUBLIC)
     PHP_FE_END
 };
 
@@ -199,7 +216,7 @@ static void php_swoole_socket_coro_free_object(zend_object *object)
 
 static zend_object* php_swoole_socket_coro_create_object(zend_class_entry *ce)
 {
-    socket_coro *sock = (socket_coro *) ecalloc(1, sizeof(socket_coro) + zend_object_properties_size(ce));
+    socket_coro *sock = (socket_coro *) zend_object_alloc(sizeof(socket_coro), ce);
     zend_object_std_init(&sock->std, ce);
     /* Even if you don't use properties yourself you should still call object_properties_init(),
      * because extending classes may use properties. (Generally a lot of the stuff you will do is
@@ -777,6 +794,7 @@ static sw_inline void swoole_socket_coro_sync_properties(zval *zobject, socket_c
 
 static void sw_inline php_swoole_init_socket(zval *zobject, socket_coro *sock)
 {
+    sock->socket->set_zero_copy(true);
     zend_update_property_long(swoole_socket_coro_ce, zobject, ZEND_STRL("fd"), sock->socket->get_fd());
 }
 
@@ -799,6 +817,7 @@ SW_API bool php_swoole_export_socket(zval *zobject, Socket *_socket)
 
 SW_API zend_object* php_swoole_dup_socket(int fd, enum swSocket_type type)
 {
+    zval zobject;
     zend_object *object = php_swoole_socket_coro_create_object(swoole_socket_coro_ce);
     socket_coro *sock = (socket_coro *) php_swoole_socket_coro_fetch_object(object);
 
@@ -818,6 +837,9 @@ SW_API zend_object* php_swoole_dup_socket(int fd, enum swSocket_type type)
         OBJ_RELEASE(object);
         return nullptr;
     }
+    
+    ZVAL_OBJ(&zobject, object);
+    zend_update_property_long(swoole_socket_coro_ce, &zobject, ZEND_STRL("fd"), sock->socket->get_fd());
     return object;
 }
 
@@ -900,14 +922,40 @@ SW_API bool php_swoole_socket_set_protocol(Socket *sock, zval *zset)
             memcpy(sock->protocol.package_eof, str_v.val(), str_v.len());
         }
     }
+    if (php_swoole_array_get_value(vht, "open_fastcgi_protocol", ztmp))
+    {
+        #define FCGI_HEADER_LEN 8
+        #define FCGI_MAX_LENGTH 0xffff
+        sock->open_length_check = zval_is_true(ztmp);
+        sock->protocol.package_length_size = FCGI_HEADER_LEN;
+        sock->protocol.package_length_offset = 0;
+        sock->protocol.package_body_offset = 0;
+        sock->protocol.get_package_length = [](swProtocol *protocol, swSocket *conn, const char *data, uint32_t size) {
+            const uint8_t *p = (const uint8_t *) data;
+            ssize_t length = 0;
+            if (size >= FCGI_HEADER_LEN)
+            {
+                length = ((p[4] << 8) | p[5]) + p[6];
+                if (length > FCGI_MAX_LENGTH)
+                {
+                    length = -1;
+                }
+                else
+                {
+                    length += FCGI_HEADER_LEN;
+                }
+            }
+            return length;
+        };
+    }
     // open mqtt protocol
     if (php_swoole_array_get_value(vht, "open_mqtt_protocol", ztmp))
     {
         sock->open_length_check = zval_is_true(ztmp);
-        sock->protocol.package_length_size = SW_MQTT_MIN_LENGTH;
-        sock->protocol.package_length_offset = 0;
-        sock->protocol.package_body_offset = 0;
-        sock->protocol.get_package_length = swMqtt_get_package_length;
+        if (zval_is_true(ztmp))
+        {
+            swMqtt_set_protocol(&sock->protocol);
+        }
     }
     // open length check
     if (php_swoole_array_get_value(vht, "open_length_check", ztmp))
@@ -986,7 +1034,7 @@ SW_API bool php_swoole_socket_set_protocol(Socket *sock, zval *zset)
     }
     else
     {
-        sock->protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
+        sock->protocol.package_max_length = SW_INPUT_BUFFER_SIZE;
     }
 
     return ret;
@@ -1129,6 +1177,50 @@ static PHP_METHOD(swoole_socket_coro, connect)
     RETURN_TRUE;
 }
 
+static PHP_METHOD(swoole_socket_coro, checkLiveness)
+{
+    swoole_get_socket_coro(sock, ZEND_THIS);
+
+    bool liveness = sock->socket->check_liveness();
+    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    RETURN_BOOL(liveness);
+}
+
+static PHP_METHOD(swoole_socket_coro, peek)
+{
+    zend_long length = SW_BUFFER_SIZE_BIG;
+
+    ZEND_PARSE_PARAMETERS_START(0, 1)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_LONG(length)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    if (UNEXPECTED(length <= 0))
+    {
+        length = SW_BUFFER_SIZE_BIG;
+    }
+
+    swoole_get_socket_coro(sock, ZEND_THIS);
+
+    zend_string *buf = zend_string_alloc(length, 0);
+    ssize_t bytes = sock->socket->peek(ZSTR_VAL(buf), length);
+    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    if (UNEXPECTED(bytes < 0))
+    {
+        zend_string_free(buf);
+        RETURN_FALSE;
+    }
+    else if (UNEXPECTED(bytes == 0))
+    {
+        zend_string_free(buf);
+        RETURN_EMPTY_STRING();
+    }
+    else
+    {
+        RETURN_STR(sw_zend_string_recycle(buf, length, bytes));
+    }
+}
+
 static sw_inline void swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAMETERS, const bool all)
 {
     zend_long length = SW_BUFFER_SIZE_BIG;
@@ -1327,7 +1419,7 @@ static PHP_METHOD(swoole_socket_coro, sendto)
 
     swoole_get_socket_coro(sock, ZEND_THIS);
 
-    ssize_t retval = sock->socket->sendto(addr, port, data, l_data);
+    ssize_t retval = sock->socket->sendto(std::string(addr, l_addr), port, data, l_data);
     swoole_socket_coro_sync_properties(ZEND_THIS, sock);
     if (retval < 0)
     {
@@ -1391,8 +1483,8 @@ static PHP_METHOD(swoole_socket_coro, getsockname)
     }
 
     array_init(return_value);
-    add_assoc_string(return_value, "address", (char *) swConnection_get_ip(sock->socket->get_type(), &sa));
-    add_assoc_long(return_value, "port", swConnection_get_port(sock->socket->get_type(), &sa));
+    add_assoc_string(return_value, "address", (char *) swSocket_get_ip(sock->socket->get_type(), &sa));
+    add_assoc_long(return_value, "port", swSocket_get_port(sock->socket->get_type(), &sa));
 }
 
 static PHP_METHOD(swoole_socket_coro, getpeername)
@@ -1407,8 +1499,8 @@ static PHP_METHOD(swoole_socket_coro, getpeername)
     }
 
     array_init(return_value);
-    add_assoc_string(return_value, "address", (char *) swConnection_get_ip(sock->socket->get_type(), &sa));
-    add_assoc_long(return_value, "port", swConnection_get_port(sock->socket->get_type(), &sa));
+    add_assoc_string(return_value, "address", (char *) swSocket_get_ip(sock->socket->get_type(), &sa));
+    add_assoc_long(return_value, "port", swSocket_get_port(sock->socket->get_type(), &sa));
 }
 
 static PHP_METHOD(swoole_socket_coro, getOption)
@@ -1673,3 +1765,12 @@ static PHP_METHOD(swoole_socket_coro, setProtocol)
         RETURN_BOOL(php_swoole_socket_set_protocol(sock->socket, zset));
     }
 }
+
+#ifdef SW_USE_OPENSSL
+static PHP_METHOD(swoole_socket_coro, sslHandshake)
+{
+    swoole_get_socket_coro(sock, ZEND_THIS);
+
+    RETURN_BOOL(sock->socket->ssl_handshake());
+}
+#endif
