@@ -14,8 +14,7 @@
   +----------------------------------------------------------------------+
 */
 
-#include "php_swoole_cxx.h"
-#include "swoole_http.h"
+#include "swoole_http_server.h"
 
 #include <iostream>
 
@@ -30,7 +29,6 @@ extern "C"
 }
 
 #include "websocket.h"
-#include "connection.h"
 #include "base64.h"
 #include "thirdparty/swoole_http_parser.h"
 
@@ -151,7 +149,7 @@ void php_swoole_websocket_frame_unpack_ex(swString *data, zval *zframe, uchar un
 
     if (data->length < sizeof(frame.header))
     {
-        SwooleG.error = SW_ERROR_PROTOCOL_ERROR;
+        swoole_set_last_error(SW_ERROR_PROTOCOL_ERROR);
         ZVAL_FALSE(zframe);
         return;
     }
@@ -164,7 +162,7 @@ void php_swoole_websocket_frame_unpack_ex(swString *data, zval *zframe, uchar un
         swString_clear(swoole_zlib_buffer);
         if (!websocket_message_uncompress(swoole_zlib_buffer, frame.payload, frame.payload_length))
         {
-            SwooleG.error = SW_ERROR_PROTOCOL_ERROR;
+            swoole_set_last_error(SW_ERROR_PROTOCOL_ERROR);
             ZVAL_FALSE(zframe);
             return;
         }
@@ -614,17 +612,19 @@ int swoole_websocket_onMessage(swServer *serv, swEventData *req)
     int fd = req->info.fd;
     uchar flags = 0;
     zend_long opcode = 0;
+    auto primary_port = serv->listen_list->front();
 
     zval zdata;
     char frame_header[2];
+    memcpy(frame_header, &req->info.ext_flags, sizeof(frame_header));
 
-    php_swoole_get_recv_data(serv, &zdata, req, frame_header, SW_WEBSOCKET_HEADER_LEN);
+    php_swoole_get_recv_data(serv, &zdata, req);
 
     // frame info has already decoded in swWebSocket_dispatch_frame
     flags  = frame_header[0];
     opcode = frame_header[1];
 
-    if (opcode == WEBSOCKET_OPCODE_CLOSE && !serv->listen_list->open_websocket_close_frame)
+    if (opcode == WEBSOCKET_OPCODE_CLOSE && !primary_port->open_websocket_close_frame)
     {
         zval_ptr_dtor(&zdata);
         return SW_OK;
@@ -777,13 +777,13 @@ static sw_inline int swoole_websocket_server_push(swServer *serv, int fd, swStri
     swConnection *conn = swWorker_get_connection(serv, fd);
     if (!conn || conn->websocket_status < WEBSOCKET_STATUS_HANDSHAKE)
     {
-        SwooleG.error = SW_ERROR_WEBSOCKET_UNCONNECTED;
+        swoole_set_last_error(SW_ERROR_WEBSOCKET_UNCONNECTED);
         php_swoole_fatal_error(E_WARNING, "the connected client of connection[%d] is not a websocket client or closed", (int ) fd);
         return SW_ERR;
     }
 
     int ret = serv->send(serv, fd, buffer->str, buffer->length);
-    if (ret < 0 && SwooleG.error == SW_ERROR_OUTPUT_SEND_YIELD)
+    if (ret < 0 && swoole_get_last_error() == SW_ERROR_OUTPUT_SEND_YIELD)
     {
         zval _return_value;
         zval *return_value = &_return_value;
@@ -817,8 +817,6 @@ static sw_inline int swoole_websocket_server_close(swServer *serv, int fd, swStr
     }
 }
 
-extern swServer* php_swoole_server_get_and_check_server(zval *zobject);
-
 static PHP_METHOD(swoole_websocket_server, disconnect)
 {
     swServer *serv = php_swoole_server_get_and_check_server(ZEND_THIS);
